@@ -88,6 +88,13 @@ def _init_posix():
         g['CC'] += ' -arch %s' % (arch,)
         g['MACOSX_DEPLOYMENT_TARGET'] = '10.7'
 
+    # pypy only: give us control over the ABI tag in a wheel name
+    if '__pypy__' in sys.builtin_module_names:
+        for suffix, mode, type_ in imp.get_suffixes():
+            if type_ == imp.C_EXTENSION:
+                g['SOABI'] = suffix.split('.')[1]
+                break
+
     global _config_vars
     _config_vars = g
 
@@ -97,6 +104,13 @@ def _init_nt():
     g = {}
     g['EXE'] = ".exe"
     g['SO'] = [s[0] for s in imp.get_suffixes() if s[2] == imp.C_EXTENSION][0]
+    g['VERSION'] = get_python_version().replace(".", "")
+    g['BINDIR'] = os.path.dirname(os.path.abspath(sys.executable))
+
+    # pypy only: give us control over the ABI tag in a wheel name
+    if '__pypy__' in sys.builtin_module_names:
+        so_ext = imp.get_suffixes()[0][0]
+        g['SOABI']= '-'.join(so_ext.split('.')[1].split('-')[:2])
 
     global _config_vars
     _config_vars = g
