@@ -745,7 +745,7 @@ class ThreadJoinOnShutdown(BaseTestCase):
 
         def generator():
             while 1:
-                yield "genereator"
+                yield "generator"
 
         def callback():
             if callback.gen is None:
