@@ -30,7 +30,7 @@ def _get_aia_uri(certificate, nid):
     if (info == ffi.NULL):
         return None;
     if lib.sk_ACCESS_DESCRIPTION_num(info) == 0:
-        lib.AUTHORITY_INFO_ACCESS_free(info)
+        lib.sk_ACCESS_DESCRIPTION_free(info)
         return None
 
     lst = []
@@ -44,7 +44,7 @@ def _get_aia_uri(certificate, nid):
         uri = ad.location.d.uniformResourceIdentifier
         ostr = _str_with_len(uri.data, uri.length)
         lst.append(ostr)
-    lib.AUTHORITY_INFO_ACCESS_free(info)
+    lib.sk_ACCESS_DESCRIPTION_free(info)
 
     # convert to tuple or None
     if len(lst) == 0: return None
@@ -266,6 +266,7 @@ def _get_crl_dp(certificate):
     for i in range(count):
         dp = lib.sk_DIST_POINT_value(dps, i);
         if not dp.distpoint:
+            # Ignore empty DP value, CVE-2019-5010
             continue
         gns = dp.distpoint.name.fullname;
 
@@ -279,7 +280,8 @@ def _get_crl_dp(certificate):
             ouri = _str_with_len(uri.data, uri.length)
             lst.append(ouri)
 
-    lib.CRL_DIST_POINTS_free(dps);
+    if lib.OPENSSL_VERSION_NUMBER < 0x10001000:
+        lib.sk_DIST_POINT_free(dps);
 
     if len(lst) == 0: return None
     return tuple(lst)
