@@ -5,13 +5,11 @@ import sys
 
 SIMPLE_TYPE_CHARS = "cbBhHiIlLdfguzZqQPXOv?"
 
-from _ctypes.basics import _CData, _CDataMeta, cdata_from_address,\
-     CArgObject
+from _ctypes.basics import (
+    _CData, _CDataMeta, cdata_from_address, CArgObject, sizeof)
 from _ctypes.builtin import ConvMode
-from _ctypes.array import Array
+from _ctypes.array import Array, byteorder
 from _ctypes.pointer import _Pointer, as_ffi_pointer
-#from _ctypes.function import CFuncPtr # this import is moved at the bottom
-                                       # because else it's circular
 
 class NULL(object):
     pass
@@ -61,6 +59,54 @@ class GlobalPyobjContainer(object):
 
 pyobj_container = GlobalPyobjContainer()
 
+def swap_bytes(value, sizeof, typeof, get_or_set):
+    def swap_2():
+        return ((value >> 8) & 0x00FF) | ((value << 8) & 0xFF00)
+
+    def swap_4():
+        return ((value & 0x000000FF) << 24) | \
+               ((value & 0x0000FF00) << 8) | \
+               ((value & 0x00FF0000) >> 8) | \
+               ((value >> 24) & 0xFF)
+
+    def swap_8():
+        return ((value & 0x00000000000000FFL) << 56) | \
+               ((value & 0x000000000000FF00L) << 40) | \
+               ((value & 0x0000000000FF0000L) << 24) | \
+               ((value & 0x00000000FF000000L) << 8) | \
+               ((value & 0x000000FF00000000L) >> 8) | \
+               ((value & 0x0000FF0000000000L) >> 24) | \
+               ((value & 0x00FF000000000000L) >> 40) | \
+               ((value >> 56) & 0xFF)
+
+    def swap_double_float(typ):
+        from struct import pack, unpack
+        if get_or_set == 'set':
+            if sys.byteorder == 'little':
+                st = pack(''.join(['>', typ]), value)
+            else:
+                st = pack(''.join(['<', typ]), value)
+            return unpack(typ, st)[0]
+        else:
+            packed = pack(typ, value)
+            if sys.byteorder == 'little':
+                st = unpack(''.join(['>', typ]), packed)
+            else:
+                st = unpack(''.join(['<', typ]), packed)
+            return st[0]
+
+    if typeof in ('c_float', 'c_float_le', 'c_float_be'):
+        return swap_double_float('f')
+    elif typeof in ('c_double', 'c_double_le', 'c_double_be'):
+        return swap_double_float('d')
+    else:
+        if sizeof == 2:
+            return swap_2()
+        elif sizeof == 4:
+            return swap_4()
+        elif sizeof == 8:
+            return swap_8()
+
 def generic_xxx_p_from_param(cls, value):
     if value is None:
         return cls(None)
@@ -83,6 +129,7 @@ def from_param_char_p(cls, value):
 
 def from_param_void_p(cls, value):
     "used by c_void_p subclasses"
+    from _ctypes.function import CFuncPtr
     res = generic_xxx_p_from_param(cls, value)
     if res is not None:
         return res
@@ -99,6 +146,14 @@ FROM_PARAM_BY_TYPE = {
     'P': from_param_void_p,
     }
 
+CTYPES_TO_PEP3118_TABLE = {
+    'i': {2: 'h', 4: 'i', 8: 'q'},
+    'I': {2: 'H', 4: 'I', 8: 'Q'},
+    'l': {4: 'l', 8: 'q'},
+    'L': {4: 'L', 8: 'Q'},
+    '?': {1: '?', 2: 'h', 4: 'l', 8: 'q'},
+}
+
 class SimpleType(_CDataMeta):
     def __new__(self, name, bases, dct):
         try:
@@ -110,6 +165,8 @@ class SimpleType(_CDataMeta):
                     break
             else:
                 raise AttributeError("cannot find _type_ attribute")
+        if tp == 'abstract':
+            tp = 'i'
         if (not isinstance(tp, str) or
             not len(tp) == 1 or
             tp not in SIMPLE_TYPE_CHARS):
@@ -121,6 +178,11 @@ class SimpleType(_CDataMeta):
         result._ffishape_ = tp
         result._fficompositesize_ = None
         result._ffiarray = ffiarray
+        if tp in CTYPES_TO_PEP3118_TABLE:
+            pep_code = CTYPES_TO_PEP3118_TABLE[tp][_rawffi.sizeof(tp)]
+        else:
+            pep_code = tp
+        result._format = byteorder[sys.byteorder] + pep_code
         if tp == 'z':
             # c_char_p
             def _getvalue(self):
@@ -271,6 +333,32 @@ class SimpleType(_CDataMeta):
             def _as_ffi_pointer_(self, ffitype):
                 return as_ffi_pointer(self, ffitype)
             result._as_ffi_pointer_ = _as_ffi_pointer_
+        if name[-2:] != '_p' and name[-3:] not in ('_le', '_be') \
+                and name not in ('c_wchar', '_SimpleCData', 'c_longdouble', 'c_bool', 'py_object'):
+            if sys.byteorder == 'big':
+                name += '_le'
+                swapped = self.__new__(self, name, bases, dct)
+                result.__ctype_le__ = swapped
+                result.__ctype_be__ = result
+                swapped.__ctype_be__ = result
+                swapped.__ctype_le__ = swapped
+                swapped._format = '<' + pep_code
+            else:
+                name += '_be'
+                swapped = self.__new__(self, name, bases, dct)
+                result.__ctype_be__ = swapped
+                result.__ctype_le__ = result
+                swapped.__ctype_le__ = result
+                swapped.__ctype_be__ = swapped
+                swapped._format = '>' + pep_code
+            from _ctypes import sizeof
+            def _getval(self):
+                return swap_bytes(self._buffer[0], sizeof(self), name, 'get')
+            def _setval(self, value):
+                d = result()
+                d.value = value
+                self._buffer[0] = swap_bytes(d.value, sizeof(self), name, 'set')
+            swapped.value = property(_getval, _setval)
 
         return result
 
@@ -279,7 +367,8 @@ class SimpleType(_CDataMeta):
     def from_param(self, value):
         if isinstance(value, self):
             return value
-
+        if self._type_ == 'abstract':
+            raise TypeError('abstract class')
         from_param_f = FROM_PARAM_BY_TYPE.get(self._type_)
         if from_param_f:
             res = from_param_f(self, value)
@@ -308,20 +397,26 @@ class SimpleType(_CDataMeta):
     def _is_pointer_like(self):
         return self._type_ in "sPzUZXO"
 
+    def _getformat(self):
+        return self._format
+
 class _SimpleCData(_CData):
     __metaclass__ = SimpleType
-    _type_ = 'i'
+    _type_ = 'abstract'
 
     def __init__(self, value=DEFAULT_VALUE):
         if not hasattr(self, '_buffer'):
             self._buffer = self._ffiarray(1, autofree=True)
         if value is not DEFAULT_VALUE:
             self.value = value
+    _init_no_arg_ = __init__
 
     def _ensure_objects(self):
-        if self._type_ not in 'zZP':
-            assert self._objects is None
-        return self._objects
+        # No '_objects' is the common case for primitives.  Examples
+        # where there is an _objects is if _type in 'zZP', or if
+        # self comes from 'from_buffer(buf)'.  See module/test_lib_pypy/
+        # ctypes_test/test_buffers.py: test_from_buffer_keepalive.
+        return getattr(self, '_objects', None)
 
     def _getvalue(self):
         return self._buffer[0]
@@ -347,5 +442,3 @@ class _SimpleCData(_CData):
 
     def __nonzero__(self):
         return self._buffer[0] not in (0, '\x00')
-
-from _ctypes.function import CFuncPtr
