@@ -10,6 +10,7 @@ if sys.platform == "win32":
 else:
     pypy_win32_extra = []
 
+libraries=_get_openssl_libraries(sys.platform)
 ffi = build_ffi_for_binding(
     module_name="_pypy_openssl",
     module_prefix="_cffi_src.openssl.",
@@ -52,9 +53,28 @@ ffi = build_ffi_for_binding(
         "pkcs7",
         "callbacks",
     ] + pypy_win32_extra,
-    libraries=_get_openssl_libraries(sys.platform),
+    libraries=libraries,
     extra_link_args=extra_link_args(compiler_type()),
 )
 
 if __name__ == '__main__':
     ffi.compile(verbose=True)
+    if sys.platform == 'win32':
+        # copy dlls from externals to the pwd
+        # maybe we should link _ssl to libraries instead of the dlls
+        # to avoid this mess
+        import os, glob, shutil
+        path_parts = os.environ['PATH'].split(';')
+        candidates = [x for x in path_parts if 'externals' in x]
+
+        def copy_from_path(dll):
+            for c in candidates:
+                files = glob.glob(os.path.join(c, dll + '*.dll'))
+                if files:
+                    for fname in files:
+                        print('copying', fname)
+                        shutil.copy(fname, '.')
+
+        if candidates:
+            for lib in libraries:
+                copy_from_path(lib)                
