@@ -51,7 +51,7 @@ def FromWideIntObj(app, value):
     wide = tkffi.new("Tcl_WideInt*")
     if tklib.Tcl_GetWideIntFromObj(app.interp, value, wide) != tklib.TCL_OK:
         app.raiseTclError()
-    return wide[0]
+    return int(wide[0])
 
 # Only when tklib.HAVE_LIBTOMMATH!
 def FromBignumObj(app, value):
@@ -171,9 +171,8 @@ def AsObj(value):
         encoded = value.encode('utf-16')[2:]
         buf = tkffi.new("char[]", encoded)
         inbuf = tkffi.cast("Tcl_UniChar*", buf)
-        return tklib.Tcl_NewUnicodeObj(buf, len(encoded)/2)
+        return tklib.Tcl_NewUnicodeObj(inbuf, len(encoded)/2)
     if isinstance(value, TclObject):
-        tklib.Tcl_IncrRefCount(value._value)
         return value._value
 
     return AsObj(str(value))
