@@ -1,11 +1,13 @@
+from __future__ import print_function
 # NOTE: run this script with LANG=en_US.UTF-8
+# works with pip install mercurial==3.0
 
 import py
 import sys
 from collections import defaultdict
 import operator
 import re
-import mercurial.localrepo
+import mercurial.hg
 import mercurial.ui
 
 ROOT = py.path.local(__file__).join('..', '..', '..', '..')
@@ -18,11 +20,13 @@ alias = {
     'Antonio Cuni': ['antocuni', 'anto'],
     'Armin Rigo': ['arigo', 'arfigo', 'armin', 'arigato'],
     'Maciej Fijalkowski': ['fijal'],
-    'Carl Friedrich Bolz': ['cfbolz', 'cf'],
+    'Carl Friedrich Bolz-Tereick': ['Carl Friedrich Bolz', 'cfbolz', 'cf', 'cbolz'],
     'Samuele Pedroni': ['pedronis', 'samuele', 'samule'],
-    'Michael Hudson': ['mwh'],
+    'Richard Plangger': ['planrich', 'plan_rich'],
+    'Remi Meier': ['remi'],
+    'Michael Hudson-Doyle': ['mwh', 'Michael Hudson'],
     'Holger Krekel': ['hpk', 'holger krekel', 'holger', 'hufpk'],
-    "Amaury Forgeot d'Arc": ['afa'],
+    "Amaury Forgeot d'Arc": ['afa', 'amauryfa@gmail.com'],
     'Alex Gaynor': ['alex', 'agaynor'],
     'David Schneider': ['bivab', 'david'],
     'Christian Tismer': ['chris', 'christian', 'tismer',
@@ -40,7 +44,7 @@ alias = {
     'Mark Pearse': ['mwp'],
     'Toon Verwaest': ['tverwaes'],
     'Eric van Riet Paap': ['ericvrp'],
-    'Jacob Hallen': ['jacob', 'jakob'],
+    'Jacob Hallen': ['jacob', 'jakob', 'jacob hallen'],
     'Anders Lehmann': ['ale', 'anders'],
     'Bert Freudenberg': ['bert'],
     'Boris Feigin': ['boris', 'boria'],
@@ -67,21 +71,31 @@ alias = {
     'Edd Barrett': ['edd'],
     'Manuel Jacob': ['mjacob'],
     'Rami Chowdhury': ['necaris'],
-    'Stanislaw Halik':['w31rd0'],
-    'Wenzhu Man':['wenzhu man', 'wenzhuman'],
-    'Anton Gulenko':['anton gulenko', 'anton_gulenko'],
-    'Richard Lancaster':['richardlancaster'],
-    'William Leslie':['William ML Leslie'],
-    'Spenser Bauman':['Spenser Andrew Bauman'],
-    'Raffael Tfirst':['raffael.tfirst@gmail.com'],
-    'timo':['timo@eistee.fritz.box'],
-    'Jasper Schulz':['Jasper.Schulz'],
-    'Aaron Gallagher':['"Aaron Gallagher'],
-    'Yasir Suhail':['yasirs'],
+    'Stanislaw Halik': ['Stanislaw Halik', 'w31rd0'],
+    'Wenzhu Man': ['wenzhu man', 'wenzhuman'],
+    'Anton Gulenko': ['anton gulenko', 'anton_gulenko'],
+    'Richard Lancaster': ['richardlancaster'],
+    'William Leslie': ['William ML Leslie'],
+    'Spenser Bauman': ['Spenser Andrew Bauman'],
+    'Raffael Tfirst': ['raffael.tfirst@gmail.com'],
+    'timo': ['timo@eistee.fritz.box'],
+    'Jasper Schulz': ['Jasper.Schulz', 'jbs'],
+    'Aaron Gallagher': ['"Aaron Gallagher'],
+    'Yasir Suhail': ['yasirs'],
+    'Squeaky': ['squeaky'],
+    "Dodan Mihai": ['mihai.dodan@gmail.com'],
+    'Wim Lavrijsen': ['wlav'],
+    'Toon Verwaest': ['toon', 'tverwaes'],
+    'Seo Sanghyeon': ['sanxiyn'],
+    'Leonardo Santagada': ['santagada'],
+    'Laurence Tratt': ['ltratt'],
+    'Pieter Zieschang': ['pzieschang', 'p_zieschang@yahoo.de'],
+    'John Witulski': ['witulski'],
+    'Andrew Lawrence': ['andrew.lawrence@siemens.com', 'andrewjlawrence'],
     }
 
 alias_map = {}
-for name, nicks in alias.iteritems():
+for name, nicks in alias.items():
     for nick in nicks:
         alias_map[nick] = name
 
@@ -99,7 +113,8 @@ def get_more_authors(log):
         return set()
     ignore_words = ['around', 'consulting', 'yesterday', 'for a bit', 'thanks',
                     'in-progress', 'bits of', 'even a little', 'floating',
-                    'a bit', 'reviewing']
+                    'a bit', 'reviewing', 'looking', 'advising', 'partly', 'ish',
+                    'watching', 'mostly', 'jumping']
     sep_words = ['and', ';', '+', '/', 'with special  by']
     nicknames = match.group(1)
     for word in ignore_words:
@@ -118,13 +133,13 @@ def get_more_authors(log):
 
 def main(show_numbers):
     ui = mercurial.ui.ui()
-    repo = mercurial.localrepo.localrepository(ui, str(ROOT))
+    repo = mercurial.hg.repository(ui, str(ROOT).encode('utf-8'))
     authors_count = defaultdict(int)
     for i in repo:
         ctx = repo[i]
         authors = set()
-        authors.add(get_canonical_author(ctx.user()))
-        authors.update(get_more_authors(ctx.description()))
+        authors.add(get_canonical_author(ctx.user().decode('utf-8')))
+        authors.update(get_more_authors(ctx.description().decode('utf-8')))
         for author in authors:
             if author not in excluded:
                 authors_count[author] += 1
@@ -139,13 +154,13 @@ def main(show_numbers):
     ##     else:
     ##         print name
 
-    items = authors_count.items()
+    items = list(authors_count.items())
     items.sort(key=operator.itemgetter(1), reverse=True)
     for name, n in items:
         if show_numbers:
-            print '%5d %s' % (n, name)
+            print('%5d %s' % (n, name))
         else:
-            print '  ' + name
+            print('  ' + name)
 
 if __name__ == '__main__':
     show_numbers = '-n' in sys.argv
