@@ -9,7 +9,7 @@ class PythonGrammar(parser.Grammar):
 
 def _get_python_grammar():
     here = os.path.dirname(__file__)
-    fp = open(os.path.join(here, "data", "Grammar3.8"))
+    fp = open(os.path.join(here, "data", "Grammar2.7"))
     try:
         gram_source = fp.read()
     finally:
@@ -19,12 +19,19 @@ def _get_python_grammar():
 
 
 python_grammar = _get_python_grammar()
+python_grammar_no_print = python_grammar.shared_copy()
+python_grammar_no_print.keyword_ids = python_grammar_no_print.keyword_ids.copy()
+del python_grammar_no_print.keyword_ids["print"]
 
 python_grammar_revdb = python_grammar.shared_copy()
+python_grammar_no_print_revdb = python_grammar_no_print.shared_copy()
 copied_token_ids = python_grammar.token_ids.copy()
 python_grammar_revdb.token_ids = copied_token_ids
+python_grammar_no_print_revdb.token_ids = copied_token_ids
 
 metavar_token_id = pytoken.python_tokens['REVDBMETAVAR']
+# the following line affects python_grammar_no_print too, since they share the
+# dict
 del python_grammar.token_ids[metavar_token_id]
 
 class _Tokens(object):
@@ -45,9 +52,14 @@ syms._rev_lookup = rev_lookup # for debugging
 del _get_python_grammar, _Tokens, tok_name, sym_name, idx
 
 def choose_grammar(print_function, revdb):
-    assert print_function
-    if revdb:
-        return python_grammar_revdb
+    if print_function:
+        if revdb:
+            return python_grammar_no_print_revdb
+        else:
+            return python_grammar_no_print
     else:
-        return python_grammar
+        if revdb:
+            return python_grammar_revdb
+        else:
+            return python_grammar
 
