@@ -179,7 +179,7 @@ class TestTypeDef:
         self.space.appexec([w_obj], """(obj):
             assert type(obj).__hash__ is None
             err = raises(TypeError, hash, obj)
-            assert err.value.message == "'some_type' objects are unhashable"
+            assert str(err.value) == "unhashable type: 'some_type'"
             """)
 
     def test_destructor(self):
@@ -423,3 +423,10 @@ class AppTestTypeDef:
     def test_get_with_none_arg(self):
         raises(TypeError, type.__dict__['__mro__'].__get__, None)
         raises(TypeError, type.__dict__['__mro__'].__get__, None, None)
+
+    def test_builtin_readonly_property(self):
+        import sys
+        x = lambda: 5
+        e = raises(TypeError, 'x.func_globals = {}')
+        if '__pypy__' in sys.builtin_module_names:
+            assert str(e.value) == "readonly attribute 'func_globals'"
