@@ -1,8 +1,12 @@
 from pypy.interpreter.typedef import TypeDef, interp_attrproperty, GetSetProperty
 from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.error import OperationError, wrap_oserror, wrap_oserror2
+from pypy.interpreter.error import (
+    OperationError, oefmt, wrap_oserror, wrap_oserror2)
+from rpython.rlib.objectmodel import keepalive_until_here
 from rpython.rlib.rarithmetic import r_longlong
+from rpython.rlib.rposix import c_read, get_saved_errno
 from rpython.rlib.rstring import StringBuilder
+from rpython.rtyper.lltypesystem import lltype, rffi
 from os import O_RDONLY, O_WRONLY, O_RDWR, O_CREAT, O_TRUNC
 import sys, os, stat, errno
 from pypy.module._io.interp_iobase import W_RawIOBase, convert_size
@@ -12,8 +16,7 @@ def interp_member_w(name, cls, doc=None):
     def fget(space, obj):
         w_value = getattr(obj, name)
         if w_value is None:
-            raise OperationError(space.w_AttributeError,
-                                 space.wrap(name))
+            raise OperationError(space.w_AttributeError, space.newtext(name))
         else:
             return w_value
     def fset(space, obj, w_value):
@@ -21,8 +24,7 @@ def interp_member_w(name, cls, doc=None):
     def fdel(space, obj):
         w_value = getattr(obj, name)
         if w_value is None:
-            raise OperationError(space.w_AttributeError,
-                                 space.wrap(name))
+            raise OperationError(space.w_AttributeError, space.newtext(name))
         setattr(obj, name, None)
 
     return GetSetProperty(fget, fset, fdel, cls=cls, doc=doc)
@@ -32,8 +34,8 @@ O_BINARY = getattr(os, "O_BINARY", 0)
 O_APPEND = getattr(os, "O_APPEND", 0)
 
 def _bad_mode(space):
-    raise OperationError(space.w_ValueError, space.wrap(
-        "Must have exactly one of read/write/append mode"))
+    raise oefmt(space.w_ValueError,
+                "Must have exactly one of read/write/append mode")
 
 def decode_mode(space, mode):
     flags = 0
@@ -70,8 +72,7 @@ def decode_mode(space, mode):
             readable = writable = True
             plus = True
         else:
-            raise OperationError(space.w_ValueError, space.wrap(
-                "invalid mode: %s" % (mode,)))
+            raise oefmt(space.w_ValueError, "invalid mode: %s", mode)
 
     if not rwa:
         _bad_mode(space)
@@ -114,6 +115,7 @@ def new_buffersize(fd, currentsize):
             return currentsize + BIGCHUNK
     return currentsize + SMALLCHUNK
 
+
 class W_FileIO(W_RawIOBase):
     def __init__(self, space):
         W_RawIOBase.__init__(self, space)
@@ -128,23 +130,22 @@ class W_FileIO(W_RawIOBase):
     def descr_new(space, w_subtype, __args__):
         self = space.allocate_instance(W_FileIO, w_subtype)
         W_FileIO.__init__(self, space)
-        return space.wrap(self)
+        return self
 
-    @unwrap_spec(mode=str, closefd=int)
+    @unwrap_spec(mode='text', closefd=int)
     def descr_init(self, space, w_name, mode='r', closefd=True):
         if space.isinstance_w(w_name, space.w_float):
-            raise OperationError(space.w_TypeError, space.wrap(
-                "integer argument expected, got float"))
+            raise oefmt(space.w_TypeError,
+                        "integer argument expected, got float")
 
         fd = -1
         try:
             fd = space.c_int_w(w_name)
-        except OperationError, e:
+        except OperationError as e:
             pass
         else:
             if fd < 0:
-                raise OperationError(space.w_ValueError, space.wrap(
-                    "negative file descriptor"))
+                raise oefmt(space.w_ValueError, "negative file descriptor")
 
         self.readable, self.writable, self.appending, flags = decode_mode(space, mode)
 
@@ -153,7 +154,7 @@ class W_FileIO(W_RawIOBase):
             if fd >= 0:
                 try:
                     os.fstat(fd)
-                except OSError, e:
+                except OSError as e:
                     if e.errno == errno.EBADF:
                         raise wrap_oserror(space, e)
                     # else: pass
@@ -162,30 +163,30 @@ class W_FileIO(W_RawIOBase):
             else:
                 self.closefd = True
                 if not closefd:
-                    raise OperationError(space.w_ValueError, space.wrap(
-                        "Cannot use closefd=False with file name"))
+                    raise oefmt(space.w_ValueError,
+                                "Cannot use closefd=False with file name")
 
                 from pypy.module.posix.interp_posix import (
                     dispatch_filename, rposix)
                 try:
                     self.fd = dispatch_filename(rposix.open)(
                         space, w_name, flags, 0666)
-                except OSError, e:
+                except OSError as e:
                     raise wrap_oserror2(space, e, w_name,
-                                        exception_name='w_IOError')
+                                        w_exception_class=space.w_IOError)
                 finally:
                     fd_is_own = True
 
             self._dircheck(space, w_name)
-            space.setattr(self, space.wrap("name"), w_name)
+            space.setattr(self, space.newtext("name"), w_name)
 
             if self.appending:
                 # For consistent behaviour, we explicitly seek to the end of file
                 # (otherwise, it might be done only on the first write()).
                 try:
                     os.lseek(self.fd, 0, os.SEEK_END)
-                except OSError, e:
-                    raise wrap_oserror(space, e, exception_name='w_IOError')
+                except OSError as e:
+                    raise wrap_oserror(space, e, w_exception_class=space.w_IOError)
         except:
             if not fd_is_own:
                 self.fd = -1
@@ -206,7 +207,7 @@ class W_FileIO(W_RawIOBase):
             return 'wb'
 
     def descr_get_mode(self, space):
-        return space.wrap(self._mode())
+        return space.newtext(self._mode())
 
     def _closed(self, space):
         return self.fd < 0
@@ -215,19 +216,15 @@ class W_FileIO(W_RawIOBase):
         if message is None:
             message = "I/O operation on closed file"
         if self.fd < 0:
-            raise OperationError(space.w_ValueError, space.wrap(message))
+            raise OperationError(space.w_ValueError, space.newtext(message))
 
     def _check_readable(self, space):
         if not self.readable:
-            raise OperationError(
-                space.w_ValueError,
-                space.wrap("file not open for reading"))
+            raise oefmt(space.w_ValueError, "file not open for reading")
 
     def _check_writable(self, space):
         if not self.writable:
-            raise OperationError(
-                space.w_ValueError,
-                space.wrap("file not open for writing"))
+            raise oefmt(space.w_ValueError, "file not open for writing")
 
     def _close(self, space):
         if self.fd < 0:
@@ -237,9 +234,8 @@ class W_FileIO(W_RawIOBase):
 
         try:
             os.close(fd)
-        except OSError, e:
-            raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
+        except OSError as e:
+            raise wrap_oserror(space, e, w_exception_class=space.w_IOError)
 
     def close_w(self, space):
         try:
@@ -267,34 +263,34 @@ class W_FileIO(W_RawIOBase):
             return
         if stat.S_ISDIR(st.st_mode):
             raise wrap_oserror2(space, OSError(errno.EISDIR, "fstat"),
-                                w_filename, exception_name='w_IOError')
+                                w_filename, w_exception_class=space.w_IOError)
 
     @unwrap_spec(pos=r_longlong, whence=int)
     def seek_w(self, space, pos, whence=0):
         self._check_closed(space)
         try:
             pos = os.lseek(self.fd, pos, whence)
-        except OSError, e:
+        except OSError as e:
             raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
-        return space.wrap(pos)
+                                w_exception_class=space.w_IOError)
+        return space.newint(pos)
 
     def tell_w(self, space):
         self._check_closed(space)
         try:
             pos = os.lseek(self.fd, 0, 1)
-        except OSError, e:
+        except OSError as e:
             raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
-        return space.wrap(pos)
+                                w_exception_class=space.w_IOError)
+        return space.newint(pos)
 
     def readable_w(self, space):
         self._check_closed(space)
-        return space.wrap(self.readable)
+        return space.newbool(self.readable)
 
     def writable_w(self, space):
         self._check_closed(space)
-        return space.wrap(self.writable)
+        return space.newbool(self.writable)
 
     def seekable_w(self, space):
         self._check_closed(space)
@@ -311,29 +307,29 @@ class W_FileIO(W_RawIOBase):
 
     def fileno_w(self, space):
         self._check_closed(space)
-        return space.wrap(self.fd)
+        return space.newint(self.fd)
 
     def isatty_w(self, space):
         self._check_closed(space)
         try:
             res = os.isatty(self.fd)
-        except OSError, e:
-            raise wrap_oserror(space, e, exception_name='w_IOError')
-        return space.wrap(res)
+        except OSError as e:
+            raise wrap_oserror(space, e, w_exception_class=space.w_IOError)
+        return space.newbool(res)
 
     def repr_w(self, space):
         if self.fd < 0:
-            return space.wrap("<_io.FileIO [closed]>")
+            return space.newtext("<_io.FileIO [closed]>")
 
         if self.w_name is None:
-            return space.wrap(
+            return space.newtext(
                 "<_io.FileIO fd=%d mode='%s'>" % (
                     self.fd, self._mode()))
         else:
             w_repr = space.repr(self.w_name)
-            return space.wrap(
+            return space.newtext(
                 "<_io.FileIO name=%s mode='%s'>" % (
-                    space.str_w(w_repr), self._mode()))
+                    space.text_w(w_repr), self._mode()))
 
     # ______________________________________________
 
@@ -344,13 +340,13 @@ class W_FileIO(W_RawIOBase):
 
         try:
             n = os.write(self.fd, data)
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.EAGAIN:
                 return space.w_None
             raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
+                               w_exception_class=space.w_IOError)
 
-        return space.wrap(n)
+        return space.newint(n)
 
     def read_w(self, space, w_size=None):
         self._check_closed(space)
@@ -362,28 +358,52 @@ class W_FileIO(W_RawIOBase):
 
         try:
             s = os.read(self.fd, size)
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.EAGAIN:
                 return space.w_None
             raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
+                               w_exception_class=space.w_IOError)
 
-        return space.wrap(s)
+        return space.newbytes(s)
 
     def readinto_w(self, space, w_buffer):
         self._check_closed(space)
         self._check_readable(space)
         rwbuffer = space.getarg_w('w*', w_buffer)
         length = rwbuffer.getlength()
-        try:
-            buf = os.read(self.fd, length)
-        except OSError, e:
-            if e.errno == errno.EAGAIN:
-                return space.w_None
-            raise wrap_oserror(space, e,
-                               exception_name='w_IOError')
-        rwbuffer.setslice(0, buf)
-        return space.wrap(len(buf))
+
+        target_address = lltype.nullptr(rffi.CCHARP.TO)
+        if length > 64:
+            try:
+                target_address = rwbuffer.get_raw_address()
+            except ValueError:
+                pass
+
+        if not target_address:
+            # unoptimized case
+            try:
+                buf = os.read(self.fd, length)
+            except OSError as e:
+                if e.errno == errno.EAGAIN:
+                    return space.w_None
+                raise wrap_oserror(space, e,
+                                   w_exception_class=space.w_IOError)
+            self.output_slice(space, rwbuffer, 0, buf)
+            return space.newint(len(buf))
+        else:
+            # optimized case: reading more than 64 bytes into a rwbuffer
+            # with a valid raw address
+            got = c_read(self.fd, target_address, length)
+            keepalive_until_here(rwbuffer)
+            got = rffi.cast(lltype.Signed, got)
+            if got >= 0:
+                return space.newint(got)
+            else:
+                err = get_saved_errno()
+                if err == errno.EAGAIN:
+                    return space.w_None
+                e = OSError(err, "read failed")
+                raise wrap_oserror(space, e, w_exception_class=space.w_IOError)
 
     def readall_w(self, space):
         self._check_closed(space)
@@ -396,7 +416,7 @@ class W_FileIO(W_RawIOBase):
 
             try:
                 chunk = os.read(self.fd, newsize - total)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == errno.EINTR:
                     space.getexecutioncontext().checksignals()
                     continue
@@ -406,13 +426,13 @@ class W_FileIO(W_RawIOBase):
                 if e.errno == errno.EAGAIN:
                     return space.w_None
                 raise wrap_oserror(space, e,
-                                   exception_name='w_IOError')
+                                   w_exception_class=space.w_IOError)
 
             if not chunk:
                 break
             builder.append(chunk)
             total += len(chunk)
-        return space.wrap(builder.build())
+        return space.newbytes(builder.build())
 
     if sys.platform == "win32":
         def _truncate(self, size):
@@ -430,8 +450,8 @@ class W_FileIO(W_RawIOBase):
 
         try:
             self._truncate(space.r_longlong_w(w_size))
-        except OSError, e:
-            raise wrap_oserror(space, e, exception_name='w_IOError')
+        except OSError as e:
+            raise wrap_oserror(space, e, w_exception_class=space.w_IOError)
 
         return w_size
 
@@ -456,7 +476,8 @@ W_FileIO.typedef = TypeDef(
     fileno = interp2app(W_FileIO.fileno_w),
     isatty = interp2app(W_FileIO.isatty_w),
     name = interp_member_w('w_name', cls=W_FileIO),
-    closefd = interp_attrproperty('closefd', cls=W_FileIO),
+    closefd = interp_attrproperty('closefd', cls=W_FileIO,
+        wrapfn="newbool"),
     mode = GetSetProperty(W_FileIO.descr_get_mode),
     )
 
