@@ -85,7 +85,7 @@ DECODER(gb2312)
         TRYMAP_DEC(gb2312, **outbuf, c ^ 0x80, IN2 ^ 0x80) {
             NEXT(2, 1)
         }
-        else return 1;
+        else return 2;
     }
 
     return 0;
@@ -141,7 +141,7 @@ DECODER(gbk)
         REQUIRE_INBUF(2)
 
         GBK_DECODE(c, IN2, **outbuf)
-        else return 1;
+        else return 2;
 
         NEXT(2, 1)
     }
@@ -266,8 +266,10 @@ DECODER(gb18030)
             REQUIRE_INBUF(4)
             c3 = IN3;
             c4 = IN4;
-            if (c < 0x81 || c3 < 0x81 || c4 < 0x30 || c4 > 0x39)
-                return 1;
+            if (c  < 0x81 || c  > 0xFE ||
+                c3 < 0x81 || c3 > 0xFE ||
+                c4 < 0x30 || c4 > 0x39)
+                return 4;
             c -= 0x81;  c2 -= 0x30;
             c3 -= 0x81; c4 -= 0x30;
 
@@ -292,12 +294,12 @@ DECODER(gb18030)
                     continue;
                 }
             }
-            return 1;
+            return 4;
         }
 
         GBK_DECODE(c, c2, **outbuf)
         else TRYMAP_DEC(gb18030ext, **outbuf, c, c2);
-        else return 1;
+        else return 2;
 
         NEXT(2, 1)
     }
@@ -333,15 +335,17 @@ ENCODER(hz)
         DBCHAR code;
 
         if (c < 0x80) {
-            if (state->i == 0) {
-                WRITE1((unsigned char)c)
-                NEXT(1, 1)
-            }
-            else {
-                WRITE3('~', '}', (unsigned char)c)
-                NEXT(1, 3)
+            if (state->i) {
+                WRITE2('~', '}');
+                NEXT_OUT(2);
                 state->i = 0;
             }
+            WRITE1((unsigned char)c);
+            NEXT(1, 1);
+            if (c == '~') {
+                WRITE1('~');
+                NEXT_OUT(1);
+            }
             continue;
         }
 
@@ -388,19 +392,17 @@ DECODER(hz)
             unsigned char c2 = IN2;
 
             REQUIRE_INBUF(2)
-            if (c2 == '~') {
+            if (c2 == '~' && state->i == 0) {
                 WRITE1('~')
-                NEXT(2, 1)
-                continue;
             }
             else if (c2 == '{' && state->i == 0)
                 state->i = 1; /* set GB */
+            else if (c2 == '\n' && state->i == 0)
+                ; /* line-continuation */
             else if (c2 == '}' && state->i == 1)
                 state->i = 0; /* set ASCII */
-            else if (c2 == '\n')
-                ; /* line-continuation */
             else
-                return 1;
+                return 2;
             NEXT(2, 0);
             continue;
         }
@@ -419,7 +421,7 @@ DECODER(hz)
                 NEXT(2, 1)
             }
             else
-                return 1;
+                return 2;
         }
     }
 
