@@ -61,14 +61,15 @@ class AppTestRandom:
         rnd.seed(1234)
         state = rnd.getstate()
         s = repr(state)
-        assert len(s) == 7331
+        assert len(s) == 7956
+        assert s.count('L') == 625
 
     def test_seed(self):
         import _random, sys
         rnd = _random.Random()
         rnd.seed()
         different_nums = []
-        mask = sys.maxsize * 2 + 1
+        mask = sys.maxint * 2 + 1
         for obj in ["spam and eggs", 3.14, 1+2j, 'a', tuple('abc')]:
             nums = []
             for o in [obj, hash(obj) & mask, -(hash(obj) & mask)]:
@@ -85,10 +86,10 @@ class AppTestRandom:
     def test_seedargs(self):
         import _random
         rnd = _random.Random()
-        for arg in [None, 0, 1, -1, 10**20, -(10**20),
-                    3.14, 1+2j, 'a', tuple('abc'), 0xffffffffff]:
+        for arg in [None, 0, 0L, 1, 1L, -1, -1L, 10**20, -(10**20),
+                    3.14, 1+2j, 'a', tuple('abc'), 0xffffffffffL]:
             rnd.seed(arg)
-        for arg in [[1, 2, 3], dict(one=1)]:
+        for arg in [range(3), dict(one=1)]:
             raises(TypeError, rnd.seed, arg)
         raises(TypeError, rnd.seed, 1, 2)
         raises(TypeError, type(rnd), [])
@@ -98,24 +99,24 @@ class AppTestRandom:
         rnd = _random.Random()
         rnd.seed()
         state1 = rnd.getstate()
-        import time; time.sleep(0.01)
-        rnd.seed()
-        state2 = rnd.getstate()
+        import time; time.sleep(1.1)     # must be at least 1 second here
+        rnd.seed()                       # (note that random.py overrides
+        state2 = rnd.getstate()          # seed() to improve the resolution)
         assert state1 != state2
 
+    def test_jumpahead(self):
+        import sys
+        import _random
+        rnd = _random.Random()
+        rnd.jumpahead(100)
+        rnd.jumpahead(sys.maxint + 2)
+
     def test_randbits(self):
         import _random
         rnd = _random.Random()
-        for n in range(1, 10):
+        for n in range(1, 10) + range(10, 1000, 15):
             k = rnd.getrandbits(n)
             assert 0 <= k < 2 ** n
-        for n in range(10, 1000, 15):
-            k = rnd.getrandbits(n)
-            assert 0 <= k < 2 ** n
-        assert rnd.getrandbits(30) != 0 # Fails every 1e10 runs
-        assert rnd.getrandbits(0) == 0
-        raises(ValueError, rnd.getrandbits, -1)
-
 
     def test_subclass(self):
         import _random
@@ -131,12 +132,3 @@ class AppTestRandom:
         rnd = _random.Random(-3**31)
         x = rnd.random()
         assert x == 0.8181851342382107
-
-        rnd = _random.Random(23)
-        x = rnd.getrandbits(15)
-        assert x == 30305
-        x = rnd.getrandbits(64)
-        assert x == 17498688041408372654
-        x = rnd.getrandbits(1000)
-        assert x == 7853306986850247776154838978003111681903290767754178881472967951394642454293814053922268204822337224228382717987163860072072211540796042432365942518253892663330370311555959936030836699709599651063582158137533337405488381774503601280079968480579765012148668221201232526424837774516436871086966847306285
-
