@@ -5,11 +5,14 @@ from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import interp_attrproperty
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
+from pypy.interpreter.unicodehelper import wcharpsize2utf8
+from pypy.interpreter.unicodehelper import wrap_unicode_out_of_range_error
 
 from rpython.rlib.clibffi import *
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.tool import rffi_platform
 from rpython.rlib.unroll import unrolling_iterable
+from rpython.rlib import rutf8
 from rpython.rlib.objectmodel import specialize
 import rpython.rlib.rposix as rposix
 
@@ -92,7 +95,7 @@ if _MS_WINDOWS:
     LL_TYPEMAP['v'] = rffi.SHORT
 
 def letter2tp(space, key):
-    from pypy.module._rawffi.array import PRIMITIVE_ARRAY_TYPES
+    from pypy.module._rawffi.interp_array import PRIMITIVE_ARRAY_TYPES
     try:
         return PRIMITIVE_ARRAY_TYPES[key]
     except KeyError:
@@ -123,7 +126,7 @@ def unpack_shape_with_length(space, w_shape):
         try:
             result = shape._array_shapes[length]
         except KeyError:
-            from pypy.module._rawffi.array import W_Array
+            from pypy.module._rawffi.interp_array import W_Array
             if isinstance(shape, W_Array) and length == 1:
                 result = shape
             else:
@@ -236,7 +239,11 @@ def open_cdll(space, name):
     except OSError as e:
         raise wrap_oserror(space, e)
 
-@unwrap_spec(name='fsencode_or_none')
+if _MS_WINDOWS:
+    name_spec = 'fsencode'
+else:
+    name_spec = 'fsencode_or_none'
+@unwrap_spec(name=name_spec)
 def descr_new_cdll(space, w_type, name):
     cdll = open_cdll(space, name)
     return W_CDLL(space, name, cdll)
@@ -353,7 +360,7 @@ class W_DataInstance(W_Root):
         self.ll_buffer = rffi.ptradd(self.ll_buffer, n)
 
     def byptr(self, space):
-        from pypy.module._rawffi.array import ARRAY_OF_PTRS
+        from pypy.module._rawffi.interp_array import ARRAY_OF_PTRS
         array = ARRAY_OF_PTRS.allocate(space, 1)
         array.setitem(space, 0, self)
         return array
@@ -416,13 +423,13 @@ def unwrap_value(space, push_func, add_arg, argdesc, letter, w_arg):
         val = s[0]
         push_func(add_arg, argdesc, val)
     elif letter == 'u':
-        s = space.unicode_w(w_arg)
-        if len(s) != 1:
+        s, lgt = space.utf8_len_w(w_arg)
+        if lgt != 1:
             raise oefmt(space.w_TypeError,
                         "Expected unicode string of length one as wide "
                         "character")
-        val = s[0]
-        push_func(add_arg, argdesc, val)
+        val = rutf8.codepoint_at_pos(s, 0)
+        push_func(add_arg, argdesc, rffi.cast(rffi.WCHAR_T, val))
     else:
         for c in unroll_letters_for_numbers:
             if letter == c:
@@ -447,7 +454,13 @@ def wrap_value(space, func, add_arg, argdesc, letter):
             elif c == 'c':
                 return space.newbytes(func(add_arg, argdesc, ll_type))
             elif c == 'u':
-                return space.newunicode(func(add_arg, argdesc, ll_type))
+                code = ord(func(add_arg, argdesc, ll_type))
+                try:
+                    return space.newutf8(rutf8.unichr_as_utf8(
+                        r_uint(code), allow_surrogates=True), 1)
+                except rutf8.OutOfRange:
+                    raise oefmt(space.w_ValueError,
+                        "unicode character %d out of range", code)
             elif c == 'f' or c == 'd' or c == 'g':
                 return space.newfloat(float(func(add_arg, argdesc, ll_type)))
             else:
@@ -472,7 +485,7 @@ class W_FuncPtr(W_Root):
         return space.newint(rffi.cast(lltype.Unsigned, self.ptr.funcsym))
 
     def byptr(self, space):
-        from pypy.module._rawffi.array import ARRAY_OF_PTRS
+        from pypy.module._rawffi.interp_array import ARRAY_OF_PTRS
         array = ARRAY_OF_PTRS.allocate(space, 1)
         array.setitem(space, 0, self.getbuffer(space))
         if tracker.DO_TRACING:
@@ -482,7 +495,7 @@ class W_FuncPtr(W_Root):
         return array
 
     def call(self, space, args_w):
-        from pypy.module._rawffi.array import W_ArrayInstance
+        from pypy.module._rawffi.interp_array import W_ArrayInstance
         from pypy.module._rawffi.structure import W_StructureInstance
         from pypy.module._rawffi.structure import W_Structure
         argnum = len(args_w)
@@ -594,11 +607,14 @@ def wcharp2unicode(space, address, maxlength=-1):
     if address == 0:
         return space.w_None
     wcharp_addr = rffi.cast(rffi.CWCHARP, address)
-    if maxlength == -1:
-        s = rffi.wcharp2unicode(wcharp_addr)
-    else:
-        s = rffi.wcharp2unicoden(wcharp_addr, maxlength)
-    return space.newunicode(s)
+    try:
+        if maxlength == -1:
+            s, lgt = rffi.wcharp2utf8(wcharp_addr)
+        else:
+            s, lgt = rffi.wcharp2utf8n(wcharp_addr, maxlength)
+    except rutf8.OutOfRange as e:
+        raise wrap_unicode_out_of_range_error(space, e)
+    return space.newutf8(s, lgt)
 
 @unwrap_spec(address=r_uint, maxlength=int)
 def charp2rawstring(space, address, maxlength=-1):
@@ -611,8 +627,8 @@ def charp2rawstring(space, address, maxlength=-1):
 def wcharp2rawunicode(space, address, maxlength=-1):
     if maxlength == -1:
         return wcharp2unicode(space, address)
-    s = rffi.wcharpsize2unicode(rffi.cast(rffi.CWCHARP, address), maxlength)
-    return space.newunicode(s)
+    s = wcharpsize2utf8(space, rffi.cast(rffi.CWCHARP, address), maxlength)
+    return space.newutf8(s, maxlength)
 
 @unwrap_spec(address=r_uint, newcontent='bufferstr')
 def rawstring2charp(space, address, newcontent):
@@ -645,7 +661,7 @@ def set_errno(space, w_errno):
 
 if sys.platform == 'win32':
     # see also
-    # https://bitbucket.org/pypy/pypy/issue/1944/ctypes-on-windows-getlasterror
+    # issue #1944 ctypes-on-windows-getlasterror
     def get_last_error(space):
         return space.newint(rwin32.GetLastError_alt_saved())
     @unwrap_spec(error=int)
@@ -653,7 +669,7 @@ if sys.platform == 'win32':
         rwin32.SetLastError_alt_saved(error)
 else:
     # always have at least a dummy version of these functions
-    # (https://bugs.pypy.org/issue1242)
+    # issue 1242
     def get_last_error(space):
         return space.newint(0)
     @unwrap_spec(error=int)
