@@ -16,7 +16,6 @@ class AppTestFfi:
         #include "src/precommondefs.h"
         #include <stdlib.h>
         #include <stdio.h>
-        #include <stdarg.h>
         #include <errno.h>
 
         struct x
@@ -224,22 +223,6 @@ class AppTestFfi:
             return old_errno;
         }
         #endif
-
-        RPY_EXPORTED
-        long variadic_sum(long n, ...) {
-            va_list ptr;
-            int sum = 0;
-            printf("n: %ld\\n", n);
-
-            va_start(ptr, n);
-            for (int i = 0; i < n; i++) {
-                long foo = va_arg(ptr, long);
-                sum += foo;
-                printf("Arg %d: %ld\\n", i, foo);
-            }
-            va_end(ptr);
-            return sum;
-        }
         '''))
         eci = ExternalCompilationInfo(include_dirs=[cdir])
         return str(platform.compile([c_file], eci, 'x', standalone=False))
@@ -262,7 +245,6 @@ class AppTestFfi:
         cls.w_sizes_and_alignments = space.wrap(dict(
             [(k, (v.c_size, v.c_alignment)) for k,v in TYPEMAP.iteritems()]))
         cls.w_float_typemap = space.wrap(TYPEMAP_FLOAT_LETTERS)
-        cls.w_is64bit = space.wrap(sys.maxint > 2147483647)
 
     def test_libload(self):
         import _rawffi
@@ -273,7 +255,7 @@ class AppTestFfi:
         try:
             _rawffi.CDLL("xxxxx_this_name_does_not_exist_xxxxx")
         except OSError as e:
-            print(e)
+            print e
             assert str(e).startswith(
                 "Cannot load library xxxxx_this_name_does_not_exist_xxxxx: ")
         else:
@@ -327,8 +309,7 @@ class AppTestFfi:
             intptr = B(1)
             intptr[0] = i
             res = get_char(dupaptr, intptr)
-            char = b'dupa'[i:i+1]
-            assert res[0] == char
+            assert res[0] == 'dupa'[i]
             intptr.free()
         dupaptr.free()
         dupa.free()
@@ -391,7 +372,7 @@ class AppTestFfi:
         a = A(6, u'xx\x00\x00xx')
         for i in (-1, 6):
             res = _rawffi.wcharp2unicode(a.buffer, i)
-            assert isinstance(res, str)
+            assert isinstance(res, unicode)
             assert res == u'xx'
         a.free()
 
@@ -401,12 +382,8 @@ class AppTestFfi:
         a = A(10, b'x'*10)
         _rawffi.rawstring2charp(a.buffer, b"foobar")
         assert b''.join([a[i] for i in range(10)]) == b"foobarxxxx"
-        _rawffi.rawstring2charp(a.buffer, memoryview(b"baz"))
+        _rawffi.rawstring2charp(a.buffer, buffer("baz"))
         assert b''.join([a[i] for i in range(10)]) == b"bazbarxxxx"
-        _rawffi.rawstring2charp(a.buffer, memoryview(b"ABCDEF"), 2)
-        assert b''.join([a[i] for i in range(10)]) == b"CDEFarxxxx"
-        _rawffi.rawstring2charp(a.buffer, memoryview(b"ZYXWVU"), 2, 3)
-        assert b''.join([a[i] for i in range(10)]) == b"XWVFarxxxx"
         a.free()
 
     def test_raw_callable(self):
@@ -716,6 +693,7 @@ class AppTestFfi:
         raises(ValueError, "lib.ptr('get_char', [], 'x')")
         raises(ValueError, "_rawffi.Structure(['x1', 'xx'])")
         raises(ValueError, _rawffi.Structure, [('x1', 'xx')])
+        raises(TypeError, _rawffi.Structure, [(u'\xe9', 'l')])
         raises(ValueError, "_rawffi.Array('xx')")
 
     def test_longs_ulongs(self):
@@ -741,7 +719,7 @@ class AppTestFfi:
         ll_to_sort = _rawffi.Array('i')(4)
         for i in range(4):
             ll_to_sort[i] = 4-i
-        qsort = libc.ptr('qsort', ['P', 'Z', 'Z', 'P'], None)
+        qsort = libc.ptr('qsort', ['P', 'l', 'l', 'P'], None)
         bogus_args = []
         def compare(a, b):
             a1 = _rawffi.Array('i').fromaddress(_rawffi.Array('P').fromaddress(a, 1)[0], 1)
@@ -752,11 +730,11 @@ class AppTestFfi:
                 return 1
             return -1
         a1 = ll_to_sort.byptr()
-        a2 = _rawffi.Array('Z')(1)
+        a2 = _rawffi.Array('l')(1)
         a2[0] = len(ll_to_sort)
-        a3 = _rawffi.Array('Z')(1)
+        a3 = _rawffi.Array('l')(1)
         a3[0] = struct.calcsize('i')
-        cb = _rawffi.CallbackPtr(compare, ['P', 'P'], 'i')
+        cb = _rawffi.CallbackPtr(compare, ['P', 'P'], 'l')
         a4 = cb.byptr()
         qsort(a1, a2, a3, a4)
         res = [ll_to_sort[i] for i in range(len(ll_to_sort))]
@@ -801,7 +779,7 @@ class AppTestFfi:
 
     def test_raising_callback(self):
         import _rawffi, sys
-        from io import StringIO
+        from StringIO import StringIO
         lib = _rawffi.CDLL(self.lib_name)
         err = StringIO()
         orig = sys.stderr
@@ -819,7 +797,7 @@ class AppTestFfi:
             val = err.getvalue()
             assert 'ZeroDivisionError' in val
             assert 'callback' in val
-            assert res[0] == 0
+            assert res[0] == 0L
         finally:
             sys.stderr = orig
 
@@ -839,7 +817,7 @@ class AppTestFfi:
 
     def test_sizes_and_alignments(self):
         import _rawffi
-        for k, (s, a) in self.sizes_and_alignments.items():
+        for k, (s, a) in self.sizes_and_alignments.iteritems():
             assert _rawffi.sizeof(k) == s
             assert _rawffi.alignment(k) == a
 
@@ -951,7 +929,7 @@ class AppTestFfi:
         a = _rawffi.Array('b')(1)
         a[0] = -5
         assert a[0] == -5
-        a[0] = 123
+        a[0] = 123L
         assert a[0] == 123
         a[0] = 0x97817182ab128111111111111171817d042
         assert a[0] == 0x42
@@ -964,7 +942,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('B')(1)
-        a[0] = 123
+        a[0] = 123L
         assert a[0] == 123
         a[0] = 0x18329b1718b97d89b7198db817d042
         assert a[0] == 0x42
@@ -977,7 +955,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('h')(1)
-        a[0] = 123
+        a[0] = 123L
         assert a[0] == 123
         a[0] = 0x9112cbc91bd91db19aaaaaaaaaaaaaa8170d42
         assert a[0] == 0x0d42
@@ -990,7 +968,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('H')(1)
-        a[0] = 123
+        a[0] = 123L
         assert a[0] == 123
         a[0] = 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042
         assert a[0] == 0xd042
@@ -1000,7 +978,7 @@ class AppTestFfi:
 
         maxptr = (256 ** struct.calcsize("P")) - 1
         a = _rawffi.Array('P')(1)
-        a[0] = 123
+        a[0] = 123L
         assert a[0] == 123
         a[0] = 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042
         assert a[0] == 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042 & maxptr
@@ -1044,8 +1022,8 @@ class AppTestFfi:
         raises(_rawffi.SegfaultException, a.__setitem__, 3, 3)
 
     def test_stackcheck(self):
-        if self.platform != "msvc" or self.is64bit:
-            skip("32-bit win32 msvc specific")
+        if self.platform != "msvc":
+            skip("win32 msvc specific")
 
         # Even if the call corresponds to the specified signature,
         # the STDCALL calling convention may detect some errors
@@ -1056,7 +1034,7 @@ class AppTestFfi:
         try:
             f()
         except ValueError as e:
-            assert "Procedure called with not enough arguments" in str(e)
+            assert "Procedure called with not enough arguments" in e.message
         else:
             assert 0, "Did not raise"
 
@@ -1067,7 +1045,7 @@ class AppTestFfi:
         try:
             f(arg)
         except ValueError as e:
-            assert "Procedure called with too many arguments" in str(e)
+            assert "Procedure called with too many arguments" in e.message
         else:
             assert 0, "Did not raise"
         arg.free()
@@ -1155,16 +1133,16 @@ class AppTestFfi:
         import _rawffi
         S = _rawffi.Structure((40, 1))
         s = S(autofree=True)
-        b = memoryview(s)
+        b = buffer(s)
         assert len(b) == 40
-        b[4] = ord(b'X')
+        b[4] = b'X'
         b[:3] = b'ABC'
         assert b[:6] == b'ABC\x00X\x00'
 
         A = _rawffi.Array('c')
         a = A(10, autofree=True)
         a[3] = b'x'
-        b = memoryview(a)
+        b = buffer(a)
         assert len(b) == 10
         assert b[3] == b'x'
         b[6] = b'y'
@@ -1183,6 +1161,11 @@ class AppTestFfi:
         import _rawffi
         S = _rawffi.Structure((40, 1))
         s = S(autofree=True)
+        addr = buffer(s)._pypy_raw_address()
+        assert type(addr) is int
+        assert buffer(s)._pypy_raw_address() == addr
+        assert buffer(s, 10)._pypy_raw_address() == addr + 10
+
         addr = memoryview(s)._pypy_raw_address()
         assert type(addr) is int
         assert memoryview(s)._pypy_raw_address() == addr
@@ -1244,17 +1227,6 @@ class AppTestFfi:
         assert z == 43
         arg.free()
 
-    def test_char_array_int(self):
-        import _rawffi
-        A = _rawffi.Array('c')
-        a = A(1)
-        a[0] = b'a'
-        assert a[0] == b'a'
-        # also accept int but return bytestring
-        a[0] = 100
-        assert a[0] == b'd'
-        a.free()
-
     def test_cdll_name(self):
         import _rawffi
         lib = _rawffi.CDLL(self.lib_name)
@@ -1278,23 +1250,6 @@ class AppTestFfi:
             raises(ValueError, _rawffi.wcharp2rawunicode, arg.itemaddress(0), 1)
         arg.free()
 
-    def test_variadic_sum(self):
-        import _rawffi
-        lib = _rawffi.CDLL(self.lib_name)
-
-        A = _rawffi.Array('l')
-        n = A(1)
-        arg1 = A(1)
-        arg2 = A(1)
-        n[0] = 2
-        arg1[0] = 3
-        arg2[0] = 15
-        variadic_sum = lib.ptr('variadic_sum', ['l', 'l', 'l'], 'l', variadic_args=2)
-        res = variadic_sum(n, arg1, arg2)
-        assert res[0] == 3 + 15
-        arg1.free()
-        arg2.free()
-        n.free()
 
 class AppTestAutoFree:
     spaceconfig = dict(usemodules=['_rawffi', 'struct'])
