@@ -4,6 +4,8 @@ import os
 import py
 from py.test import raises, skip
 from pypy.interpreter.gateway import app2interp_temp
+from pypy.module._sre import interp_sre
+from rpython.rlib.rsre.test import support
 
 
 def init_app_test(cls, space):
@@ -20,6 +22,37 @@ def init_app_test(cls, space):
             sys.path.pop(0)
         """)
 
+def _test_sre_ctx_(self, str, start, end):
+    # Use the MatchContextForTests class, which handles Position
+    # instances instead of plain integers.  This is used to detect when
+    # we're accepting or escaping a Position to app-level, which we
+    # should not: Positions are meant to be byte indexes inside a
+    # possibly UTF8 string, not character indexes.
+    if not isinstance(start, support.Position):
+        start = support.Position(start)
+    if not isinstance(end, support.Position):
+        end = support.Position(end)
+    return support.MatchContextForTests(str, start, end)
+
+def _bytepos_to_charindex(self, bytepos):
+    if isinstance(self.ctx, support.MatchContextForTests):
+        return self.ctx._real_pos(bytepos)
+    return _org_maker[1](self, bytepos)
+
+def setup_module(mod):
+    mod._org_maker = (
+        interp_sre.W_SRE_Pattern._make_str_match_context,
+        interp_sre.W_SRE_Match.bytepos_to_charindex,
+        )
+    interp_sre.W_SRE_Pattern._make_str_match_context = _test_sre_ctx_
+    interp_sre.W_SRE_Match.bytepos_to_charindex = _bytepos_to_charindex
+
+def teardown_module(mod):
+    (
+        interp_sre.W_SRE_Pattern._make_str_match_context,
+        interp_sre.W_SRE_Match.bytepos_to_charindex,
+    ) = mod._org_maker
+
 
 class AppTestSrePy:
     def test_magic(self):
@@ -86,6 +119,7 @@ class AppTestSrePattern:
         assert ["a", "u"] == re.findall("b(.)", "abalbus")
         assert [("a", "l"), ("u", "s")] == re.findall("b(.)(.)", "abalbus")
         assert [("a", ""), ("s", "s")] == re.findall("b(a|(s))", "babs")
+        assert ['', '', '', ''] == re.findall("X??", "1X4")   # changes in 3.7
 
     def test_findall_unicode(self):
         import re
@@ -109,11 +143,19 @@ class AppTestSrePattern:
         assert ['', 'a', 'l', 'a', 'lla'] == re.split("b(a)", "balballa")
         assert ['', 'a', None, 'l', 'u', None, 'lla'] == (
             re.split("b([ua]|(s))", "balbulla"))
+        assert ["abc"] == re.split("", "abc")
+        assert ["abc"] == re.split("X?", "abc")
+        assert ["a", "c"] == re.split("b?", "abc")
 
     def test_weakref(self):
         import re, _weakref
         _weakref.ref(re.compile(r""))
 
+    def test_match_compat(self):
+        import re
+        res = re.match(r'(a)|(b)', 'b').start(1)
+        assert res == -1
+
 
 class AppTestSreMatch:
     spaceconfig = dict(usemodules=('array', ))
@@ -222,6 +264,7 @@ class AppTestSreMatch:
         assert "rbd\nbr\n" == re.sub("a(.)", r"b\1\n", "radar")
         assert ("rbd\nbr\n", 2) == re.subn("a(.)", r"b\1\n", "radar")
         assert ("bbbba", 2) == re.subn("a", "b", "ababa", 2)
+        assert "XaXbXcX" == re.sub("", "X", "abc")
 
     def test_sub_unicode(self):
         import re
@@ -278,6 +321,10 @@ class AppTestSreMatch:
         import re
         assert re.sub('=\w{2}', 'x', '=CA') == 'x'
 
+    def test_sub_emptymatch(self):
+        import re
+        assert re.sub(r"b*", "*", "abc") == "*a*c*"   # changes in 3.7
+
     def test_match_array(self):
         import re, array
         a = array.array('c', 'hello')
@@ -361,9 +408,8 @@ class AppTestSreScanner:
         assert "a" == p.match().group(0)
         assert "a" == p.match().group(0)
         assert None == p.match()
-        assert "a" == p.match().group(0)
-        assert "a" == p.match().group(0)
-        assert None == p.match()
+        # the rest has been changed somewhere between Python 2.6.9
+        # and Python 2.7.18.  PyPy now follows the 2.7.18 behavior
         assert None == p.match()
         assert None == p.match()
 
@@ -382,6 +428,13 @@ class AppTestSreScanner:
         assert ("bla", "") == (p.search().group(0), p.search().group(0))
         assert None == p.search()
 
+    def test_scanner_empty_match(self):
+        import re, sys
+        p = re.compile("a??").scanner("bac")
+        assert ("", "", "", "") == (p.search().group(0), p.search().group(0),
+                                    p.search().group(0), p.search().group(0))
+        assert None == p.search()
+
 
 class AppTestGetlower:
     spaceconfig = dict(usemodules=('_locale',))
@@ -1026,6 +1079,9 @@ class AppTestUnicodeExtra:
         import re
         match = re.search(u"\u1234", u"\u1233\u1234\u1235")
         assert match.string == u"\u1233\u1234\u1235"
+        # check ascii version too
+        match = re.search(u"a", u"bac")
+        assert match.string == u"bac"
 
     def test_match_start(self):
         import re
