@@ -12,11 +12,11 @@ try:
     priv_flags = win32security.TOKEN_ADJUST_PRIVILEGES | win32security.TOKEN_QUERY
     hToken = win32security.OpenProcessToken (win32api.GetCurrentProcess (), priv_flags)
     privilege_id = win32security.LookupPrivilegeValue (None, "SeBackupPrivilege")
-    win32security.AdjustTokenPrivileges (hToken, 0, [(privilege_id, win32security.SE_PRIVILEGE_ENABLED)])
+    ret = win32security.AdjustTokenPrivileges (hToken, 0, [(privilege_id, win32security.SE_PRIVILEGE_ENABLED)])
 except:
     canSaveKey = False
 else:
-    canSaveKey = True
+    canSaveKey = len(ret) > 0
 
 class AppTestHKey:
     spaceconfig = dict(usemodules=('_winreg',))
@@ -127,7 +127,7 @@ class AppTestFfi:
         import errno
         try:
             QueryInfoKey(0)
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             assert e.winerror == 6
             assert e.errno == errno.EBADF
             # XXX translations...
@@ -151,25 +151,30 @@ class AppTestFfi:
 
     def test_readValues(self):
         from _winreg import OpenKey, EnumValue, QueryValueEx, EnumKey
+        from _winreg import REG_SZ, REG_EXPAND_SZ
         key = OpenKey(self.root_key, self.test_key_name)
         sub_key = OpenKey(key, "sub_key")
         index = 0
         while 1:
             try:
                 data = EnumValue(sub_key, index)
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 break
             assert data in self.test_data
             index = index + 1
         assert index == len(self.test_data)
 
         for name, value, type in self.test_data:
-            assert QueryValueEx(sub_key, name) == (value, type)
+            result = QueryValueEx(sub_key, name)
+            assert result == (value, type)
+            if type == REG_SZ or type == REG_EXPAND_SZ:
+                assert isinstance(result[0], unicode)     # not string
 
         assert EnumKey(key, 0) == "sub_key"
         raises(EnvironmentError, EnumKey, key, 1)
 
     def test_delete(self):
+        # must be run after test_SetValueEx
         from _winreg import OpenKey, KEY_ALL_ACCESS, DeleteValue, DeleteKey
         key = OpenKey(self.root_key, self.test_key_name, 0, KEY_ALL_ACCESS)
         sub_key = OpenKey(key, "sub_key", 0, KEY_ALL_ACCESS)
@@ -185,6 +190,7 @@ class AppTestFfi:
         h.Close()
 
     def test_savekey(self):
+        # must be run after test_SetValueEx
         if not self.canSaveKey:
             skip("CPython needs win32api to set the SeBackupPrivilege security privilege")
         from _winreg import OpenKey, KEY_ALL_ACCESS, SaveKey
@@ -227,7 +233,7 @@ class AppTestFfi:
         from _winreg import EnumValue, QueryValueEx, HKEY_PERFORMANCE_DATA
         try:
             EnumValue(HKEY_PERFORMANCE_DATA, 0)
-        except WindowsError, e:
+        except WindowsError as e:
             import errno
             if e.errno in (errno.EPERM, errno.EACCES):
                 skip("access denied to registry key "
