@@ -1,11 +1,13 @@
 from rpython.rtyper.lltypesystem import lltype, rffi
+from rpython.rlib.rbigint import rbigint, InvalidSignednessError
 from pypy.module.cpyext.api import (
     cpython_api, PyObject, build_type_checkers_flags, Py_ssize_t,
     CONST_STRING, ADDR, CANNOT_FAIL, INTP_real)
 from pypy.objspace.std.longobject import W_LongObject
 from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.unicodehelper import wcharpsize2utf8
 from pypy.module.cpyext.intobject import PyInt_AsUnsignedLongMask
-from rpython.rlib.rbigint import rbigint, InvalidSignednessError
+from pypy.module.cpyext.pyerrors import PyErr_BadInternalCall
 
 PyLong_Check, PyLong_CheckExact = build_type_checkers_flags("Long")
 
@@ -96,10 +98,14 @@ def PyLong_AsUnsignedLongLong(space, w_long):
     Return a C unsigned long representation of the contents of pylong.
     If pylong is greater than ULONG_MAX, an OverflowError is
     raised."""
+    if not w_long:
+        return PyErr_BadInternalCall(space)
     try:
         return rffi.cast(rffi.ULONGLONG, space.r_ulonglong_w(w_long))
     except OperationError as e:
         if e.match(space, space.w_ValueError):
+            if not w_long:
+                raise
             e.w_type = space.w_OverflowError
         raise
 
@@ -109,6 +115,8 @@ def PyLong_AsUnsignedLongLongMask(space, w_long):
     PyLongObject, if it is not already one, and then return its value as
     unsigned long long, without checking for overflow.
     """
+    if w_long is None:
+        raise oefmt(space.w_SystemError, "value is NULL")
     num = space.bigint_w(w_long)
     return num.ulonglongmask()
 
@@ -191,7 +199,7 @@ def PyLong_FromUnicode(space, u, length, base):
     string, length gives the number of characters, and base is the radix
     for the conversion.  The radix must be in the range [2, 36]; if it is
     out of range, ValueError will be raised."""
-    w_value = space.newunicode(rffi.wcharpsize2unicode(u, length))
+    w_value = space.newutf8(wcharpsize2utf8(space, u, length), length)
     w_base = space.newint(rffi.cast(lltype.Signed, base))
     return space.call_function(space.w_long, w_value, w_base)
 
