@@ -1,11 +1,14 @@
 
 from rpython.rlib import rerased, jit
+from rpython.rlib.objectmodel import keepalive_until_here
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.objspace.std.listobject import (
     ListStrategy, UNROLL_CUTOFF, W_ListObject, ObjectListStrategy)
 from pypy.module.cpyext.api import (
-    cpython_api, CANNOT_FAIL, CONST_STRING, Py_ssize_t, PyObject, PyObjectP)
+    cpython_api, CANNOT_FAIL, CONST_STRING, Py_ssize_t, PyObject, PyObjectP,
+    generic_cpy_call)
 from pypy.module.cpyext.pyobject import PyObject, make_ref, from_ref
+from pypy.module.cpyext.pyobject import as_pyobj, incref
 from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.objspace.std import tupleobject
 
@@ -19,7 +22,7 @@ def PySequence_Repeat(space, w_obj, count):
     """Return the result of repeating sequence object o count times, or NULL on
     failure.  This is the equivalent of the Python expression o * count.
     """
-    return space.mul(w_obj, space.wrap(count))
+    return space.mul(w_obj, space.newint(count))
 
 @cpython_api([PyObject], rffi.INT_real, error=CANNOT_FAIL)
 def PySequence_Check(space, w_obj):
@@ -47,47 +50,56 @@ def PySequence_Fast(space, w_obj, m):
     converted to a sequence, and raises a TypeError, raise a new TypeError with
     m as the message text. If the conversion otherwise, fails, reraise the
     original exception"""
+    if isinstance(w_obj, tupleobject.W_AbstractTupleObject):
+        return w_obj   # CCC avoid the double conversion that occurs here
     if isinstance(w_obj, W_ListObject):
-        # make sure we can return a borrowed obj from PySequence_Fast_GET_ITEM    
-        w_obj.convert_to_cpy_strategy(space)
+        # note: we used to call w_obj.convert_to_cpy_strategy() here,
+        # but we really have to call it from PySequence_Fast_GET_ITEM()
+        # because some people never call PySequence_Fast() if they know
+        # the object is a list.
         return w_obj
     try:
-        return W_ListObject.newlist_cpyext(space, space.listview(w_obj))
+        return tupleobject.W_TupleObject(space.fixedview(w_obj))
     except OperationError as e:
         if e.match(space, space.w_TypeError):
-            raise OperationError(space.w_TypeError, space.wrap(rffi.charp2str(m)))
+            raise OperationError(space.w_TypeError, space.newtext(rffi.charp2str(m)))
         raise e
 
-@cpython_api([rffi.VOIDP, Py_ssize_t], PyObject, result_borrowed=True)
-def PySequence_Fast_GET_ITEM(space, w_obj, index):
+# CCC this should be written as a C macro, at least for the tuple case
+@cpython_api([rffi.VOIDP, Py_ssize_t], PyObject, result_is_ll=True)
+def PySequence_Fast_GET_ITEM(space, py_obj, index):
     """Return the ith element of o, assuming that o was returned by
     PySequence_Fast(), o is not NULL, and that i is within bounds.
     """
-    if isinstance(w_obj, W_ListObject):
-        return w_obj.getitem(index)
-    elif isinstance(w_obj, tupleobject.W_TupleObject):
-        return w_obj.wrappeditems[index]
-    raise oefmt(space.w_TypeError,
-                "PySequence_Fast_GET_ITEM called but object is not a list or "
-                "sequence")
+    py_obj = rffi.cast(PyObject, py_obj)
+    if PyTuple_Check(space, py_obj):
+        from pypy.module.cpyext.tupleobject import PyTupleObject
+        py_tuple = rffi.cast(PyTupleObject, py_obj)
+        return py_tuple.c_ob_item[index]
+    else:
+        from pypy.module.cpyext.listobject import PyList_GET_ITEM
+        w_obj = from_ref(space, py_obj)
+        return PyList_GET_ITEM(space, w_obj, index)
 
 @cpython_api([rffi.VOIDP], Py_ssize_t, error=CANNOT_FAIL)
-def PySequence_Fast_GET_SIZE(space, w_obj):
+def PySequence_Fast_GET_SIZE(space, py_obj):
     """Returns the length of o, assuming that o was returned by
     PySequence_Fast() and that o is not NULL.  The size can also be
     gotten by calling PySequence_Size() on o, but
     PySequence_Fast_GET_SIZE() is faster because it can assume o is a list
     or tuple."""
-    if isinstance(w_obj, W_ListObject):
-        return w_obj.length()
-    elif isinstance(w_obj, tupleobject.W_TupleObject):
-        return len(w_obj.wrappeditems)
-    raise oefmt(space.w_TypeError,
-                "PySequence_Fast_GET_SIZE called but object is not a list or "
-                "sequence")
+    py_obj = rffi.cast(PyObject, py_obj)
+    if PyTuple_Check(space, py_obj):
+        from pypy.module.cpyext.tupleobject import PyTupleObject
+        py_tuple = rffi.cast(PyTupleObject, py_obj)
+        return py_tuple.c_ob_size
+    else:
+        from pypy.module.cpyext.listobject import PyList_GET_SIZE
+        w_obj = from_ref(space, py_obj)
+        return PyList_GET_SIZE(space, w_obj)
 
 @cpython_api([rffi.VOIDP], PyObjectP)
-def PySequence_Fast_ITEMS(space, w_obj):
+def PySequence_Fast_ITEMS(space, py_obj):
     """Return the underlying array of PyObject pointers.  Assumes that o was returned
     by PySequence_Fast() and o is not NULL.
 
@@ -95,35 +107,39 @@ def PySequence_Fast_ITEMS(space, w_obj):
     So, only use the underlying array pointer in contexts where the sequence
     cannot change.
     """
-    if isinstance(w_obj, W_ListObject):
-        cpy_strategy = space.fromcache(CPyListStrategy)
-        if w_obj.strategy is cpy_strategy:
-            return w_obj.get_raw_items() # asserts it's a cpyext strategy
-    raise oefmt(space.w_TypeError,
-                "PySequence_Fast_ITEMS called but object is not the result of "
-                "PySequence_Fast")
+    py_obj = rffi.cast(PyObject, py_obj)
+    if PyTuple_Check(space, py_obj):
+        from pypy.module.cpyext.tupleobject import PyTupleObject
+        py_tuple = rffi.cast(PyTupleObject, py_obj)
+        return rffi.cast(PyObjectP, py_tuple.c_ob_item)
+    else:
+        from pypy.module.cpyext.listobject import get_list_storage
+        w_obj = from_ref(space, py_obj)
+        assert isinstance(w_obj, W_ListObject)
+        storage = get_list_storage(space, w_obj)
+        return rffi.cast(PyObjectP, storage._elems)
 
 @cpython_api([PyObject, Py_ssize_t, Py_ssize_t], PyObject)
 def PySequence_GetSlice(space, w_obj, start, end):
     """Return the slice of sequence object o between i1 and i2, or NULL on
     failure. This is the equivalent of the Python expression o[i1:i2]."""
-    return space.getslice(w_obj, space.wrap(start), space.wrap(end))
+    return space.getslice(w_obj, space.newint(start), space.newint(end))
 
 @cpython_api([PyObject, Py_ssize_t, Py_ssize_t, PyObject], rffi.INT_real, error=-1)
 def PySequence_SetSlice(space, w_obj, start, end, w_value):
     """Assign the sequence object v to the slice in sequence object o from i1 to
     i2.  This is the equivalent of the Python statement o[i1:i2] = v."""
-    space.setslice(w_obj, space.wrap(start), space.wrap(end), w_value)
+    space.setslice(w_obj, space.newint(start), space.newint(end), w_value)
     return 0
 
 @cpython_api([PyObject, Py_ssize_t, Py_ssize_t], rffi.INT_real, error=-1)
 def PySequence_DelSlice(space, w_obj, start, end):
     """Delete the slice in sequence object o from i1 to i2.  Returns -1 on
     failure.  This is the equivalent of the Python statement del o[i1:i2]."""
-    space.delslice(w_obj, space.wrap(start), space.wrap(end))
+    space.delslice(w_obj, space.newint(start), space.newint(end))
     return 0
 
-@cpython_api([rffi.VOIDP, Py_ssize_t], PyObject)
+@cpython_api([rffi.VOIDP, Py_ssize_t], PyObject, result_is_ll=True)
 def PySequence_ITEM(space, w_obj, i):
     """Return the ith element of o or NULL on failure. Macro form of
     PySequence_GetItem() but without checking that
@@ -132,12 +148,36 @@ def PySequence_ITEM(space, w_obj, i):
 
     This function used an int type for i. This might require
     changes in your code for properly supporting 64-bit systems."""
-    return space.getitem(w_obj, space.wrap(i))
+    # XXX we should call Py*_GET_ITEM() instead of Py*_GetItem()
+    # from here, but we cannot because we are also called from
+    # PySequence_GetItem()
+    py_obj = as_pyobj(space, w_obj)
+    if isinstance(w_obj, tupleobject.W_TupleObject):
+        from pypy.module.cpyext.tupleobject import PyTuple_GetItem
+        py_res = PyTuple_GetItem(space, py_obj, i)
+        incref(space, py_res)
+        keepalive_until_here(w_obj)
+        return py_res
+    if isinstance(w_obj, W_ListObject):
+        from pypy.module.cpyext.listobject import PyList_GetItem
+        py_res = PyList_GetItem(space, py_obj, i)
+        incref(space, py_res)
+        keepalive_until_here(w_obj)
+        return py_res
+    as_sequence = py_obj.c_ob_type.c_tp_as_sequence
+    if as_sequence and as_sequence.c_sq_item:
+        ret = generic_cpy_call(space, as_sequence.c_sq_item, w_obj, i)
+        return make_ref(space, ret)
+    w_ret = space.getitem(w_obj, space.newint(i))
+    return make_ref(space, w_ret)
 
-@cpython_api([PyObject, Py_ssize_t], PyObject)
+@cpython_api([PyObject, Py_ssize_t], PyObject, result_is_ll=True)
 def PySequence_GetItem(space, w_obj, i):
     """Return the ith element of o, or NULL on failure. This is the equivalent of
     the Python expression o[i]."""
+    if i < 0:
+        l = PySequence_Length(space, w_obj)
+        i += l
     return PySequence_ITEM(space, w_obj, i)
 
 @cpython_api([PyObject], PyObject)
@@ -175,7 +215,7 @@ def PySequence_InPlaceRepeat(space, w_o, count):
 
     This function used an int type for count. This might require
     changes in your code for properly supporting 64-bit systems."""
-    return space.inplace_mul(w_o, space.wrap(count))
+    return space.inplace_mul(w_o, space.newint(count))
 
 
 @cpython_api([PyObject, PyObject], rffi.INT_real, error=-1)
@@ -203,14 +243,14 @@ def PySequence_SetItem(space, w_o, i, w_v):
     if PyDict_Check(space, w_o) or not PySequence_Check(space, w_o):
         raise oefmt(space.w_TypeError,
                     "'%T' object does not support item assignment", w_o)
-    space.setitem(w_o, space.wrap(i), w_v)
+    space.setitem(w_o, space.newint(i), w_v)
     return 0
 
 @cpython_api([PyObject, Py_ssize_t], rffi.INT_real, error=-1)
 def PySequence_DelItem(space, w_o, i):
     """Delete the ith element of object o.  Returns -1 on failure.  This is the
     equivalent of the Python statement del o[i]."""
-    space.delitem(w_o, space.wrap(i))
+    space.delitem(w_o, space.newint(i))
     return 0
 
 @cpython_api([PyObject, PyObject], Py_ssize_t, error=-1)
@@ -230,14 +270,14 @@ def PySequence_Index(space, w_seq, w_obj):
             if e.match(space, space.w_StopIteration):
                 break
             raise
-        if space.is_true(space.eq(w_next, w_obj)):
+        if space.eq_w(w_next, w_obj):
             return idx
         idx += 1
 
     raise oefmt(space.w_ValueError, "sequence.index(x): x not in sequence")
 
 class CPyListStrategy(ListStrategy):
-    erase, unerase = rerased.new_erasing_pair("empty")
+    erase, unerase = rerased.new_erasing_pair("cpylist")
     erase = staticmethod(erase)
     unerase = staticmethod(unerase)
 
@@ -256,17 +296,14 @@ class CPyListStrategy(ListStrategy):
     def setitem(self, w_list, index, w_obj):
         storage = self.unerase(w_list.lstorage)
         index = self._check_index(index, storage._length)
-        decref(w_list.space, storage._elems[index])
+        py_old = storage._elems[index]
         storage._elems[index] = make_ref(w_list.space, w_obj)
+        decref(w_list.space, py_old)
 
     def length(self, w_list):
         storage = self.unerase(w_list.lstorage)
         return storage._length
 
-    def get_raw_items(self, w_list):
-        storage = self.unerase(w_list.lstorage)
-        return storage._elems
-
     def getslice(self, w_list, start, stop, step, length):
         w_list.switch_to_object_strategy()
         return w_list.strategy.getslice(w_list, start, stop, step, length)
@@ -293,6 +330,23 @@ class CPyListStrategy(ListStrategy):
     def getitems_fixedsize(self, w_list):
         return self.getitems_unroll(w_list)
 
+    def copy_into(self, w_list, w_other):
+        w_other.strategy = self
+        w_other.lstorage = self.getstorage_copy(w_list)
+
+    def clone(self, w_list):
+        storage = self.getstorage_copy(w_list)
+        w_clone = W_ListObject.from_storage_and_strategy(self.space, storage,
+                                                         self)
+        return w_clone
+
+    def getitems_copy(self, w_list):
+        return self.getitems(w_list) # getitems copies anyway
+
+    def getstorage_copy(self, w_list):
+        lst = self.getitems(w_list)
+        return self.erase(CPyListStorage(w_list.space, lst))
+
     #------------------------------------------
     # all these methods fail or switch strategy and then call ListObjectStrategy's method
 
@@ -300,23 +354,9 @@ class CPyListStrategy(ListStrategy):
         w_list.switch_to_object_strategy()
         w_list.strategy.setslice(w_list, start, stop, step, length)
 
-    def get_sizehint(self):
-        return -1
-
     def init_from_list_w(self, w_list, list_w):
         raise NotImplementedError
 
-    def clone(self, w_list):
-        storage = w_list.lstorage  # lstorage is tuple, no need to clone
-        w_clone = W_ListObject.from_storage_and_strategy(self.space, storage,
-                                                         self)
-        w_clone.switch_to_object_strategy()
-        return w_clone
-        
-    def copy_into(self, w_list, w_other):
-        w_list.switch_to_object_strategy()
-        w_list.strategy.copy_into(w_list, w_other)
-
     def _resize_hint(self, w_list, hint):
         pass
 
@@ -324,13 +364,6 @@ class CPyListStrategy(ListStrategy):
         w_list.switch_to_object_strategy()
         return w_list.strategy.find(w_list, w_item, start, stop)
 
-    def getitems_copy(self, w_list):
-        w_list.switch_to_object_strategy()
-        return w_list.strategy.getitems_copy(w_list)
-
-    def getstorage_copy(self, w_list):
-        raise NotImplementedError
-
     def append(self, w_list, w_item):
         w_list.switch_to_object_strategy()
         w_list.strategy.append(w_list, w_item)
@@ -345,11 +378,11 @@ class CPyListStrategy(ListStrategy):
 
     def pop(self, w_list, index):
         w_list.switch_to_object_strategy()
-        w_list.strategy.pop(w_list, index)
+        return w_list.strategy.pop(w_list, index)
 
     def pop_end(self, w_list):
         w_list.switch_to_object_strategy()
-        w_list.strategy.pop_end(w_list)
+        return w_list.strategy.pop_end(w_list)
 
     def insert(self, w_list, index, w_item):
         w_list.switch_to_object_strategy()
@@ -377,7 +410,7 @@ class CPyListStrategy(ListStrategy):
 
     def is_empty_strategy(self):
         return False
-        
+
 
 PyObjectList = lltype.Ptr(lltype.Array(PyObject, hints={'nolength': True}))
 
