@@ -5,7 +5,7 @@ from pypy.interpreter import executioncontext
 from pypy.interpreter.executioncontext import ExecutionContext
 from rpython.rtyper.annlowlevel import llhelper
 from rpython.rlib.rdynload import DLLHANDLE
-from rpython.rlib import rawrefcount
+from rpython.rlib import rawrefcount, rgil
 import sys
 
 
@@ -45,6 +45,13 @@ class State:
         # XXX will leak if _PyDateTime_Import already called
         self.datetimeAPI = []
 
+        self.cpyext_is_imported = False
+
+    def make_sure_cpyext_is_imported(self):
+        if not self.cpyext_is_imported:
+            self.space.getbuiltinmodule("cpyext")    # mandatory to init cpyext
+            self.cpyext_is_imported = True
+
     def set_exception(self, operror):
         self.clear_exception()
         ec = self.space.getexecutioncontext()
@@ -188,15 +195,7 @@ class State:
 
     @specialize.arg(1)
     def ccall(self, name, *args):
-        from pypy.module.cpyext.api import cpyext_glob_tid_ptr
-        # This is similar to doing a direct call to state.C.PyXxx(), but
-        # must be used for any function that might potentially call back
-        # RPython code---most of them can, e.g. PyErr_NoMemory().
-        assert cpyext_glob_tid_ptr[0] == 0
-        cpyext_glob_tid_ptr[0] = -1
-        result = getattr(self.C, name)(*args)
-        cpyext_glob_tid_ptr[0] = 0
-        return result
+        return getattr(self.C, name)(*args)
 
 
 class CNamespace:
