@@ -2,10 +2,10 @@ from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.cpyext.structmemberdefs import *
-from pypy.module.cpyext.api import ADDR, PyObjectP, cpython_api, CONST_STRING
+from pypy.module.cpyext.api import PyObjectP, cpython_api, CONST_STRING
 from pypy.module.cpyext.intobject import PyInt_AsLong, PyInt_AsUnsignedLong
 from pypy.module.cpyext.pyerrors import PyErr_Occurred
-from pypy.module.cpyext.pyobject import PyObject, Py_DecRef, from_ref, make_ref
+from pypy.module.cpyext.pyobject import PyObject, decref, from_ref, make_ref
 from pypy.module.cpyext.bytesobject import (
     PyString_FromString, PyString_FromStringAndSize)
 from pypy.module.cpyext.floatobject import PyFloat_AsDouble
@@ -36,9 +36,7 @@ _HEADER = 'pypy_structmember_decl.h'
 
 @cpython_api([CONST_STRING, lltype.Ptr(PyMemberDef)], PyObject, header=_HEADER)
 def PyMember_GetOne(space, obj, w_member):
-    addr = rffi.cast(ADDR, obj)
-    addr += w_member.c_offset
-
+    addr = rffi.ptradd(obj, w_member.c_offset)
     member_type = rffi.cast(lltype.Signed, w_member.c_type)
     for converter in integer_converters:
         typ, lltyp, _ = converter
@@ -89,8 +87,7 @@ def PyMember_GetOne(space, obj, w_member):
 @cpython_api([rffi.CCHARP, lltype.Ptr(PyMemberDef), PyObject], rffi.INT_real,
              error=-1, header=_HEADER)
 def PyMember_SetOne(space, obj, w_member, w_value):
-    addr = rffi.cast(ADDR, obj)
-    addr += w_member.c_offset
+    addr = rffi.ptradd(obj, w_member.c_offset)
     member_type = rffi.cast(lltype.Signed, w_member.c_type)
     flags = rffi.cast(lltype.Signed, w_member.c_flags)
 
@@ -123,7 +120,7 @@ def PyMember_SetOne(space, obj, w_member, w_value):
     elif member_type in [T_OBJECT, T_OBJECT_EX]:
         array = rffi.cast(PyObjectP, addr)
         if array[0]:
-            Py_DecRef(space, array[0])
+            decref(space, array[0])
         array[0] = make_ref(space, w_value)
     else:
         raise oefmt(space.w_SystemError, "bad memberdescr type")
