@@ -9,6 +9,12 @@ static PyMethodDef methods[] = {
     { NULL }
 };
 
+#ifdef __GNUC__
+extern __attribute__((visibility("default")))
+#else
+extern __declspec(dllexport)
+#endif
+
 PyMODINIT_FUNC
 initmodinit(void) {
     Py_InitModule3("modinit", methods, "");
