@@ -17,7 +17,7 @@ class TestWeakReference(BaseApiTest):
     def test_proxy(self, space, api):
         w_obj = space.w_Warning  # some weakrefable object
         w_proxy = api.PyWeakref_NewProxy(w_obj, None)
-        assert space.unwrap(space.str(w_proxy)) == "<class 'Warning'>"
+        assert space.unwrap(space.str(w_proxy)) == "<type 'exceptions.Warning'>"
         assert space.unwrap(space.repr(w_proxy)).startswith('<weak')
 
     def test_weakref_lockobject(self, space, api):
