@@ -32,30 +32,29 @@ class AppTestFcntl:
 
         f = open(self.tmp + "b", "w+")
 
-        original = fcntl.fcntl(f, 1, 0)
+        fcntl.fcntl(f, 1, 0)
         fcntl.fcntl(f, 1)
-        fcntl.fcntl(F(int(f.fileno())), 1)
+        fcntl.fcntl(F(long(f.fileno())), 1)
         raises(TypeError, fcntl.fcntl, "foo")
         raises(TypeError, fcntl.fcntl, f, "foo")
         exc = raises(TypeError, fcntl.fcntl, F("foo"), 1)
         assert str(exc.value) == 'fileno() returned a non-integer'
-        exc = raises(OverflowError, fcntl.fcntl, 2147483647 + 1, 1, 0)
-        exc = raises(OverflowError, fcntl.fcntl, F(2147483647 + 1), 1, 0)
-        exc = raises(OverflowError, fcntl.fcntl, -2147483648 - 1, 1, 0)
-        exc = raises(OverflowError, fcntl.fcntl, F(-2147483648 - 1), 1, 0)
+        exc = raises(ValueError, fcntl.fcntl, 2147483647 + 1, 1, 0)
+        assert str(exc.value) == 'file descriptor cannot be a negative integer (-1)'
+        exc = raises(ValueError, fcntl.fcntl, F(2147483647 + 1), 1, 0)
+        assert str(exc.value) == 'file descriptor cannot be a negative integer (-1)'
+        exc = raises(ValueError, fcntl.fcntl, -2147483648 - 1, 1, 0)
+        assert str(exc.value) == 'file descriptor cannot be a negative integer (-1)'
+        exc = raises(ValueError, fcntl.fcntl, F(-2147483648 - 1), 1, 0)
+        assert str(exc.value) == 'file descriptor cannot be a negative integer (-1)'
         raises(ValueError, fcntl.fcntl, -1, 1, 0)
         raises(ValueError, fcntl.fcntl, F(-1), 1, 0)
-        raises(ValueError, fcntl.fcntl, F(int(-1)), 1, 0)
-        assert fcntl.fcntl(f, 1, 0) == original
-        assert fcntl.fcntl(f, 2, "foo") == b"foo"
-        assert fcntl.fcntl(f, 2, b"foo") == b"foo"
-
-        # This is supposed to work I think, but CPython 3.5 refuses it
-        # for reasons I don't understand:
-        #     >>> _testcapi.getargs_s_hash(memoryview(b"foo"))
-        #     TypeError: must be read-only bytes-like object, not memoryview
-        #
-        # assert fcntl.fcntl(f, 2, memoryview(b"foo")) == b"foo"
+        raises(ValueError, fcntl.fcntl, F(long(-1)), 1, 0)
+        assert fcntl.fcntl(f, 1, 0) == 0
+        assert fcntl.fcntl(f, 2, "foo") == "foo"
+        assert fcntl.fcntl(f, 2, buffer("foo")) == "foo"
+        exc = raises(TypeError, fcntl.fcntl, f, 2, memoryview("foo"))
+        assert 'integer' in str(exc.value)
 
         try:
             os.O_LARGEFILE
@@ -209,7 +208,7 @@ class AppTestFcntl:
         raises(TypeError, fcntl.ioctl, "foo")
         raises(TypeError, fcntl.ioctl, 0, "foo")
         #raises(TypeError, fcntl.ioctl, 0, TIOCGPGRP, float(0))
-        raises(TypeError, fcntl.ioctl, 0, TIOCGPGRP, 1, "foo", "bar")
+        raises(TypeError, fcntl.ioctl, 0, TIOCGPGRP, 1, "foo")
 
         child_pid, mfd = pty.fork()
         if child_pid == 0:
@@ -232,17 +231,21 @@ class AppTestFcntl:
             assert buf.tostring() == expected
 
             buf = array.array('i', [0])
-            res = fcntl.ioctl(mfd, TIOCGPGRP, memoryview(buf))
-            assert res == 0
-            assert buf.tostring() == expected
+            res = fcntl.ioctl(mfd, TIOCGPGRP, buffer(buf))
+            assert res == expected
+            assert buf.tostring() == '\x00' * 4
 
-            raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, (), False)
+            exc = raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, memoryview('abc'))
+            assert 'integer' in str(exc.value)
+            exc = raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, buffer(buf), False)
+            assert str(exc.value) == "ioctl requires a file or file descriptor, an integer and optionally an integer or buffer argument"
+            exc = raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, memoryview('abc'), False)
+            assert str(exc.value) == "ioctl requires a file or file descriptor, an integer and optionally an integer or buffer argument"
 
             res = fcntl.ioctl(mfd, TIOCGPGRP, buf, False)
             assert res == expected
 
-            # xxx this fails on CPython 3.5, that's a minor bug
-            #raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, "\x00\x00", True)
+            raises(TypeError, fcntl.ioctl, mfd, TIOCGPGRP, "\x00\x00", True)
 
             res = fcntl.ioctl(mfd, TIOCGPGRP, "\x00\x00\x00\x00")
             assert res == expected
@@ -277,7 +280,7 @@ class AppTestFcntl:
         try:
             if termios.TIOCSWINSZ < 0:
                 set_winsz_opcode_maybe_neg = termios.TIOCSWINSZ
-                set_winsz_opcode_pos = termios.TIOCSWINSZ & 0xffffffff
+                set_winsz_opcode_pos = termios.TIOCSWINSZ & 0xffffffffL
             else:
                 set_winsz_opcode_pos = termios.TIOCSWINSZ
                 set_winsz_opcode_maybe_neg, = struct.unpack("i",
@@ -291,29 +294,13 @@ class AppTestFcntl:
             os.close(mfd)
             os.close(sfd)
 
-    def test_ioctl_use_mask_on_op(self):
-        import os
-        import fcntl
-        import pty
-        try:
-            from termios import TCFLSH, TCIOFLUSH
-        except ImportError:
-            skip("don't know how to test ioctl() on this platform")
-
-        mfd, sfd = pty.openpty()
-        try:
-            assert fcntl.ioctl(mfd, TCFLSH | 0x1000000000000000000000, TCIOFLUSH) == 0
-        finally:
-            os.close(mfd)
-            os.close(sfd)
-
     def test_large_flag(self):
         import sys
         if any(plat in sys.platform
                for plat in ('darwin', 'openbsd', 'freebsd')):
             skip("Mac OS doesn't have any large flag in fcntl.h")
         import fcntl, sys
-        if sys.maxsize == 2147483647:
+        if sys.maxint == 2147483647:
             assert fcntl.DN_MULTISHOT == -2147483648
         else:
             assert fcntl.DN_MULTISHOT == 2147483648
