@@ -1,43 +1,69 @@
-# -*- encoding: utf-8 -*-
-import inspect
-import os
+
+import marshal
+import py, os
 import time
+import struct
+from pypy.module.imp.importing import get_pyc_magic, _w_long
+from StringIO import StringIO
+
+from rpython.tool.udir import udir
 from zipfile import ZIP_STORED
 
-from pypy.interpreter.test.test_fsencode import BaseFSEncodeTest
-from rpython.tool.udir import udir
 
-
-class AppTestZipimport(BaseFSEncodeTest):
+class AppTestZipimport:
     """ A bit structurized tests stolen and adapted from
     cpy's regression tests
     """
     compression = ZIP_STORED
     spaceconfig = {
-        "usemodules": ['zipimport', 'time', 'struct', 'binascii', 'marshal'],
+        "usemodules": ['zipimport', 'time', 'struct', 'itertools', 'binascii'],
     }
     pathsep = os.path.sep
 
+    @classmethod
+    def make_pyc(cls, space, co, mtime):
+        data = marshal.dumps(co)
+        if type(mtime) is type(0.0):
+            # Mac mtimes need a bit of special casing
+            if mtime < 0x7fffffff:
+                mtime = int(mtime)
+            else:
+                mtime = int(-0x100000000L + long(mtime))
+        s = StringIO()
+        try:
+            _w_long(s, get_pyc_magic(space))
+        except AttributeError:
+            import imp
+            s.write(imp.get_magic())
+        pyc = s.getvalue() + struct.pack("<i", int(mtime)) + data
+        return pyc
+
     @classmethod
     def make_class(cls):
-        BaseFSEncodeTest.setup_class.im_func(cls)
+        # XXX: this is (mostly) wrong: .compile() compiles the code object
+        # using the host python compiler, but then in the tests we load it
+        # with py.py. It works (mostly by chance) because the two functions
+        # are very simple and the bytecodes are compatible enough.
+        co = py.code.Source("""
+        def get_name():
+            return __name__
+        def get_file():
+            return __file__
+        """).compile()
+
         space = cls.space
-        w = space.wrap
 
-        cls.w_appdirect = w(cls.runappdirect)
-        cls.w_now = w(time.time())
-        cls.w_compression = w(cls.compression)
-        cls.w_pathsep = w(cls.pathsep)
-        cls.tmpdir = udir.ensure('zipimport_%s_%s' % (__name__, cls.__name__),
-                                 dir=1)
-        ziptestmodule = cls.tmpdir.join("somezip.zip")
-        cls.w_tmpzip = w(str(ziptestmodule))
-
-        # Cache get_pyc()
-        get_pyc_source = inspect.getsource(
-            cls.w__get_pyc.im_func).splitlines()[1:]
-        get_pyc_source.insert(0, '    (mtime):')
-        cls.w__test_pyc = space.appexec([cls.w_now], '\n'.join(get_pyc_source))
+        tmpdir = udir.ensure('zipimport_%s_%s' % (__name__, cls.__name__), dir=1)
+        now = time.time()
+        cls.w_now = space.wrap(now)
+        test_pyc = cls.make_pyc(space, co, now)
+        cls.w_test_pyc = space.wrap(test_pyc)
+        cls.w_compression = space.wrap(cls.compression)
+        cls.w_pathsep = space.wrap(cls.pathsep)
+        #ziptestmodule = tmpdir.ensure('ziptestmodule.zip').write(
+        ziptestmodule = tmpdir.join("somezip.zip")
+        cls.w_tmpzip = space.wrap(str(ziptestmodule))
+        cls.tmpdir = tmpdir
 
     def setup_class(cls):
         cls.make_class()
@@ -50,10 +76,9 @@ class AppTestZipimport(BaseFSEncodeTest):
         w_cache = space.getattr(space.getbuiltinmodule('zipimport'),
                                 space.wrap('_zip_directory_cache'))
         space.call_function(space.getattr(w_cache, space.wrap('clear')))
-        self.w_modules = space.call_function(
-            space.w_list,
+        self.w_modules = space.call_method(
             space.getattr(space.getbuiltinmodule('sys'),
-                          space.wrap('modules')))
+                          space.wrap('modules')), 'copy')
 
     def teardown_method(self, meth):
         space = self.space
@@ -70,33 +95,6 @@ class AppTestZipimport(BaseFSEncodeTest):
         """)
         self.w_modules = []
 
-    def w_get_pyc(self):
-        # always create the pyc on the host under appdirect, otherwise
-        # the pre-made copy is fine
-        return self._get_pyc(self.now) if self.appdirect else self._test_pyc
-
-    def w__get_pyc(self, mtime):
-        import imp
-        import marshal
-
-        if type(mtime) is float:
-            # Mac mtimes need a bit of special casing
-            if mtime < 0x7fffffff:
-                mtime = int(mtime)
-            else:
-                mtime = int(-0x100000000 + int(mtime))
-        mtimeb = int(mtime).to_bytes(4, 'little', signed=True)
-
-        source = """\
-def get_name():
-    return __name__
-def get_file():
-    return __file__"""
-        data = marshal.dumps(compile(source, 'uuu.py', 'exec'))
-        size = len(data).to_bytes(4, 'little', signed=True)
-
-        return imp.get_magic() + b"\0\0\0\0" + mtimeb + size + data
-
     def w_now_in_the_future(self, delta):
         self.now += delta
 
@@ -170,7 +168,7 @@ def get_file():
 
     def test_pyc(self):
         import sys, os
-        self.writefile("uuu.pyc", self.get_pyc())
+        self.writefile("uuu.pyc", self.test_pyc)
         self.writefile("uuu.py", "def f(x): return x")
         mod = __import__('uuu', globals(), locals(), [])
         expected = {
@@ -188,34 +186,14 @@ def get_file():
         import zipimport
         z = zipimport.zipimporter(self.zipfile)
         code = z.get_code('uuu')
-        assert isinstance(code, type((lambda:0).__code__))
-
-    def test_hash_pyc(self):
-        import sys, os
-        pyc = self.get_pyc()
-        b = bytearray(pyc)
-        b[4] = 0b1 # unchecked hash
-        self.writefile("uuu.pyc", bytes(b))
-        mod = __import__('uuu', globals(), locals(), [])
-        assert mod.__name__ == 'uuu'
-        assert mod.__file__.endswith('.zip'+os.sep+'uuu.pyc')
-
-    def test_hash_check_not_imported_pyc(self):
-        import zipimport
-        import sys, os
-        pyc = self.get_pyc()
-        b = bytearray(pyc)
-        b[4] = 0b10 # checked hash
-        self.writefile("uuu.pyc", bytes(b))
-        raises(zipimport.ZipImportError,
-            "__import__('uuu', globals(), locals(), [])")
+        assert isinstance(code, type((lambda:0).func_code))
 
     def test_bad_pyc(self):
         import zipimport
         import sys
-        m0 = self.get_pyc()[0]
+        m0 = ord(self.test_pyc[0])
         m0 ^= 0x04
-        test_pyc = bytes([m0]) + self.get_pyc()[1:]
+        test_pyc = chr(m0) + self.test_pyc[1:]
         self.writefile("xxbad_pyc.pyc", test_pyc)
         raises(zipimport.ZipImportError,
                "__import__('xxbad_pyc', globals(), locals(), [])")
@@ -232,18 +210,18 @@ def get_file():
 
     def test_force_py(self):
         import sys
-        m0 = self.get_pyc()[0]
+        m0 = ord(self.test_pyc[0])
         m0 ^= 0x04
-        test_pyc = bytes([m0]) + self.get_pyc()[1:]
+        test_pyc = chr(m0) + self.test_pyc[1:]
         self.writefile("xxforce_py.pyc", test_pyc)
         self.writefile("xxforce_py.py", "def f(x): return x")
         mod = __import__("xxforce_py", globals(), locals(), [])
         assert mod.f(3) == 3
 
     def test_sys_modules(self):
-        m0 = self.get_pyc()[0]
+        m0 = ord(self.test_pyc[0])
         m0 ^= 0x04
-        test_pyc = bytes([m0]) + self.get_pyc()[1:]
+        test_pyc = chr(m0) + self.test_pyc[1:]
         self.writefile("uuu.pyc", test_pyc)
         import sys
         import zipimport
@@ -265,8 +243,8 @@ def get_file():
 
     def test_package_bug(self):
         import os, sys
-        import types
-        mod = types.ModuleType('xxuuv')
+        import new
+        mod = new.module('xxuuv')
         mod.__path__ = [self.zipfile + '/xxuuv']
         sys.modules['xxuuv'] = mod
         #
@@ -280,13 +258,13 @@ def get_file():
 
     def test_pyc_in_package(self):
         import os, sys
-        import types
-        mod = types.ModuleType('xxuuw')
+        import new
+        mod = new.module('xxuuw')
         mod.__path__ = [self.zipfile + '/xxuuw']
         sys.modules['xxuuw'] = mod
         #
         self.writefile("xxuuw/__init__.py", "")
-        self.writefile("xxuuw/zz.pyc", self.get_pyc())
+        self.writefile("xxuuw/zz.pyc", self.test_pyc)
         mod = __import__("xxuuw.zz", globals(), locals(), ['__doc__'])
         assert mod.__file__ == (self.zipfile + os.path.sep
                                 + "xxuuw" + os.path.sep
@@ -297,8 +275,8 @@ def get_file():
     def test_functions(self):
         import os
         import zipimport
-        data = b"saddsadsa"
-        pyc_data = self.get_pyc()
+        data = "saddsadsa"
+        pyc_data = self.test_pyc
         self.now_in_the_future(+5)   # write the zipfile 5 secs after the .pyc
         self.writefile("xxx", data)
         self.writefile("xx/__init__.py", "5")
@@ -362,14 +340,11 @@ def get_file():
         """
         import os
         import zipimport
-        if self.appdirect:
-            # py3k's appdirect startup may populate _zip_directory_cache
-            zipimport._zip_directory_cache.clear()
         self.writefile("directory/package/__init__.py", "")
         importer = zipimport.zipimporter(self.zipfile + "/directory")
         l = [i for i in zipimport._zip_directory_cache]
         assert len(l) == 1
-        k = list(zipimport._zip_directory_cache[l[0]].keys())
+        k = zipimport._zip_directory_cache[l[0]].keys()
         assert k[0] == os.path.sep.join(['directory','package','__init__.py'])
 
     def test_path_hooks(self):
@@ -377,70 +352,10 @@ def get_file():
         import zipimport
         assert sys.path_hooks.count(zipimport.zipimporter) == 1
 
-    def w__make_unicode_filename(self):
-        if not self.testfn_unencodable:
-            import sys
-            skip("can't run this test with %s as filesystem encoding"
-                 % sys.getfilesystemencoding())
-        import os
-        head, tail = os.path.split(self.zipfile)
-        self.zipfile = (head + os.path.sep + tail[:4] +
-                        self.testfn_unencodable + tail[4:])
-
-    def test_unicode_filename_notfound(self):
-        if not self.special_char:
-            import sys
-            skip("can't run this test with %s as filesystem encoding"
-                 % sys.getfilesystemencoding())
-        import zipimport
-        raises(zipimport.ZipImportError,
-               zipimport.zipimporter, self.special_char)
-
-    def test_unicode_filename_invalid_zippath(self):
-        import zipimport
-        import os
-        self._make_unicode_filename()
-        os.mkdir(self.zipfile)
-        raises(zipimport.ZipImportError,
-               zipimport.zipimporter, self.zipfile)
-
-    def test_unicode_filename_invalid_zip(self):
-        import zipimport
-        self._make_unicode_filename()
-        open(self.zipfile, 'wb').write(b'invalid zip')
-        raises(zipimport.ZipImportError,
-               zipimport.zipimporter, self.zipfile)
-
-    def test_unicode_filename_existing(self):
-        import zipimport
-        self._make_unicode_filename()
-        self.writefile('ä.py', '3')
-        z = zipimport.zipimporter(self.zipfile)
-        assert not z.is_package('ä')
-        raises(ImportError, z.is_package, 'xx')
-        assert z.get_source('ä') == '3'
-        raises(ImportError, z.get_source, 'xx')
-        assert z.get_code('ä')
-        raises(ImportError, z.get_code, 'xx')
-        mod = z.load_module('ä')
-        #assert z.load_module('ä') is mod
-        assert z.get_filename('ä') == mod.__file__
-        raises(ImportError, z.load_module, 'xx')
-        raises(ImportError, z.get_filename, 'xx')
-        assert z.archive == self.zipfile
-        # PyPy fix: check null byte behavior:
-        import sys
-        if '__pypy__' in sys.builtin_module_names:
-            raises(ImportError, z.is_package, 'ä\0 b')
-            raises(ImportError, z.get_source, 'ä\0 b')
-            raises(ImportError, z.get_code, 'ä\0 b')
-            raises(ImportError, z.load_module, 'ä\0 b')
-            raises(ImportError, z.get_filename, 'ä\0 b')
-
     def test_co_filename(self):
         self.writefile('mymodule.py', """
 def get_co_filename():
-    return get_co_filename.__code__.co_filename
+    return get_co_filename.func_code.co_filename
 """)
         import os
         expected = self.zipfile + os.sep + 'mymodule.py'
@@ -455,47 +370,11 @@ def get_co_filename():
         co_filename = code.co_filename
         assert co_filename == expected
 
-    def test_unencodable(self):
-        if not self.testfn_unencodable:
-            skip("need an unencodable filename")
-        import os
-        import time
-        import zipimport
-        from zipfile import ZipFile, ZipInfo
-        filename = self.testfn_unencodable + ".zip"
-        z = ZipFile(filename, "w")
-        zinfo = ZipInfo("uu.py", time.localtime(self.now))
-        zinfo.compress_type = self.compression
-        z.writestr(zinfo, '')
-        z.close()
-        try:
-            zipimport.zipimporter(filename).load_module('uu')
-        finally:
-            os.remove(filename)
-
     def test_import_exception(self):
         self.writefile('x1test.py', '1/0')
         self.writefile('x1test/__init__.py', 'raise ValueError')
         raises(ValueError, __import__, 'x1test', None, None, [])
 
-    def test_namespace_pkg(self):
-        self.writefile('foo/', '')
-        self.writefile('foo/one.py', "attr = 'portion1 foo one'\n")
-        foo = __import__('foo.one', None, None, [])
-        assert foo.one.attr == 'portion1 foo one'
-
-    def test_namespace_pkg_nested(self):
-        self.writefile('foo/', '')
-        self.writefile('foo/bar/', '')
-        self.writefile('foo/bar/one.py', "attr = 'portion1 foo one'\n")
-        foo = __import__('foo.bar.one', None, None, [])
-        assert foo.bar.one.attr == 'portion1 foo one'
-
-    def test___spec__(self):
-        self.writefile('uvwv.py', 'spec = __spec__')
-        mod = __import__('uvwv', globals(), locals(), [])
-        assert mod.spec is not None
-
 
 if os.sep != '/':
     class AppTestNativePathSep(AppTestZipimport):
