@@ -2,9 +2,8 @@
 Tests for the zlib module.
 """
 
-import pypy
+import sys
 import py
-
 try:
     import zlib
 except ImportError:
@@ -17,6 +16,15 @@ try:
 except ImportError:
     py.test.skip("no zlib C library on this machine")
 
+def test_unsigned_to_signed_32bit():
+    assert interp_zlib.unsigned_to_signed_32bit(123) == 123
+    assert interp_zlib.unsigned_to_signed_32bit(2**31) == -2**31
+    assert interp_zlib.unsigned_to_signed_32bit(2**32-1) == -1
+    if sys.maxint > 2**32:
+        from rpython.rlib.rarithmetic import r_uint
+        assert interp_zlib.unsigned_to_signed_32bit(r_uint(sys.maxint)) == -1
+        assert interp_zlib.unsigned_to_signed_32bit(r_uint(sys.maxint+1)) == 0
+
 
 class AppTestZlib(object):
     spaceconfig = dict(usemodules=['zlib'])
@@ -31,12 +39,9 @@ class AppTestZlib(object):
         cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
 
         cls.w_zlib = cls.space.getbuiltinmodule('zlib')
-        expanded = b'some bytes which will be compressed'
-        cls.w_expanded = cls.space.newbytes(expanded)
-        cls.w_compressed = cls.space.newbytes(zlib.compress(expanded))
-        cls.w_LICENSE = cls.space.newbytes(
-            py.path.local(pypy.__file__).dirpath().dirpath()
-            .join('LICENSE').read())
+        expanded = 'some bytes which will be compressed'
+        cls.w_expanded = cls.space.wrap(expanded)
+        cls.w_compressed = cls.space.wrap(zlib.compress(expanded))
 
         def intentionally_break_a_z_stream(space, w_zobj):
             """
@@ -54,9 +59,6 @@ class AppTestZlib(object):
             interp2app(intentionally_break_a_z_stream),
         )
 
-    def test_def_buf_size(self):
-        assert self.zlib.DEF_BUF_SIZE >= 0
-
     def test_error(self):
         """
         zlib.error should be an exception class.
@@ -66,55 +68,53 @@ class AppTestZlib(object):
     def test_crc32(self):
         """
         When called with a string, zlib.crc32 should compute its CRC32 and
-        return it as an unsigned 32 bit integer.
+        return it as a signed 32 bit integer.  On 64-bit machines too
+        (it is a bug in CPython < 2.6 to return unsigned values in this case).
         """
-        assert self.zlib.crc32(b'') == 0
-        assert self.zlib.crc32(b'\0') == 3523407757
-        assert self.zlib.crc32(b'hello, world.') == 3358036098
+        assert self.zlib.crc32('') == 0
+        assert self.zlib.crc32('\0') == -771559539
+        assert self.zlib.crc32('hello, world.') == -936931198
 
     def test_crc32_start_value(self):
         """
         When called with a string and an integer, zlib.crc32 should compute the
         CRC32 of the string using the integer as the starting value.
         """
-        assert self.zlib.crc32(b'', 42) == 42
-        assert self.zlib.crc32(b'\0', 42) == 163128923
-        assert self.zlib.crc32(b'hello, world.', 42) == 1090960721
-        hello = b'hello, '
+        assert self.zlib.crc32('', 42) == 42
+        assert self.zlib.crc32('\0', 42) == 163128923
+        assert self.zlib.crc32('hello, world.', 42) == 1090960721
+        hello = 'hello, '
         hellocrc = self.zlib.crc32(hello)
-        world = b'world.'
+        world = 'world.'
         helloworldcrc = self.zlib.crc32(world, hellocrc)
         assert helloworldcrc == self.zlib.crc32(hello + world)
 
     def test_crc32_negative_start(self):
-        v = self.zlib.crc32(b'', -1)
-        assert v == 4294967295
-
-    def test_crc32_negative_start_bug(self):
-        v1 = self.zlib.crc32(b'ham', 0xffffffff)
-        v2 = self.zlib.crc32(b'ham', -1)
-        assert v1 == v2
+        v = self.zlib.crc32('', -1)
+        assert v == -1
 
     def test_crc32_negative_long_start(self):
-        v = self.zlib.crc32(b'', -1)
-        assert v == 4294967295
-        assert self.zlib.crc32(b'foo', -99999999999999999999999) == 1611238463
+        v = self.zlib.crc32('', -1L)
+        assert v == -1
+        assert self.zlib.crc32('foo', -99999999999999999999999) == 1611238463
 
     def test_crc32_long_start(self):
         import sys
-        v = self.zlib.crc32(b'', sys.maxsize*2)
-        assert v == 4294967294
-        assert self.zlib.crc32(b'foo', 99999999999999999999999) == 1635107045
+        v = self.zlib.crc32('', sys.maxint*2)
+        assert v == -2
+        assert self.zlib.crc32('foo', 99999999999999999999999) == 1635107045
 
     def test_adler32(self):
         """
         When called with a string, zlib.adler32() should compute its adler 32
-        checksum and return it as an unsigned 32 bit integer.
+        checksum and return it as a signed 32 bit integer.
+        On 64-bit machines too
+        (it is a bug in CPython < 2.6 to return unsigned values in this case).
         """
-        assert self.zlib.adler32(b'') == 1
-        assert self.zlib.adler32(b'\0') == 65537
-        assert self.zlib.adler32(b'hello, world.') == 571147447
-        assert self.zlib.adler32(b'x' * 23) == 2172062409
+        assert self.zlib.adler32('') == 1
+        assert self.zlib.adler32('\0') == 65537
+        assert self.zlib.adler32('hello, world.') == 571147447
+        assert self.zlib.adler32('x' * 23) == -2122904887
 
     def test_adler32_start_value(self):
         """
@@ -122,18 +122,18 @@ class AppTestZlib(object):
         the adler 32 checksum of the string using the integer as the starting
         value.
         """
-        assert self.zlib.adler32(b'', 42) == 42
-        assert self.zlib.adler32(b'\0', 42) == 2752554
-        assert self.zlib.adler32(b'hello, world.', 42) == 606078176
-        assert self.zlib.adler32(b'x' * 23, 42) == 2233862898
-        hello = b'hello, '
+        assert self.zlib.adler32('', 42) == 42
+        assert self.zlib.adler32('\0', 42) == 2752554
+        assert self.zlib.adler32('hello, world.', 42) == 606078176
+        assert self.zlib.adler32('x' * 23, 42) == -2061104398
+        hello = 'hello, '
         hellosum = self.zlib.adler32(hello)
-        world = b'world.'
+        world = 'world.'
         helloworldsum = self.zlib.adler32(world, hellosum)
         assert helloworldsum == self.zlib.adler32(hello + world)
 
-        assert self.zlib.adler32(b'foo', -1) == 45547858
-        assert self.zlib.adler32(b'foo', 99999999999999999999999) == 4180148562
+        assert self.zlib.adler32('foo', -1) == 45547858
+        assert self.zlib.adler32('foo', 99999999999999999999999) == -114818734
 
     def test_invalidLevel(self):
         """
@@ -159,7 +159,6 @@ class AppTestZlib(object):
         zlib.decompressobj should return an object which can be used to
         decompress bytes.
         """
-        import sys
         decompressor = self.zlib.decompressobj()
         bytes = decompressor.decompress(self.compressed)
         bytes += decompressor.flush()
@@ -184,18 +183,17 @@ class AppTestZlib(object):
         Try to feed garbage to zlib.decompress().
         """
         raises(self.zlib.error, self.zlib.decompress, self.compressed[:-2])
-        raises(self.zlib.error, self.zlib.decompress, b'foobar')
+        raises(self.zlib.error, self.zlib.decompress, 'foobar')
 
     def test_bad_arguments(self):
-        import zlib, sys
-        BIG = sys.maxsize + 1
+        import zlib
         raises(ValueError, zlib.decompressobj().flush, 0)
         raises(ValueError, zlib.decompressobj().flush, -1)
         raises(TypeError, zlib.decompressobj().flush, None)
         raises(ValueError, zlib.decompressobj().decompress, b'abc', -1)
         raises(TypeError, zlib.decompressobj().decompress, b'abc', None)
         raises(TypeError, self.zlib.decompress, self.compressed, None)
-        raises(OverflowError, self.zlib.decompress, self.compressed, BIG)
+        raises(OverflowError, self.zlib.decompress, self.compressed, 2**31)
 
     def test_empty_flush(self):
         import zlib
@@ -204,28 +202,6 @@ class AppTestZlib(object):
         dco = zlib.decompressobj()
         assert dco.flush() == b""
 
-    def test_decompress_eof(self):
-        import zlib
-        x = b'x\x9cK\xcb\xcf\x07\x00\x02\x82\x01E'  # 'foo'
-        dco = zlib.decompressobj()
-        assert dco.eof == False
-        dco.decompress(x[:-5])
-        assert dco.eof == False
-        dco.decompress(x[-5:])
-        assert dco.eof == True
-        dco.flush()
-        assert dco.eof == True
-
-    def test_decompress_eof_incomplete_stream(self):
-        import zlib
-        x = b'x\x9cK\xcb\xcf\x07\x00\x02\x82\x01E'  # 'foo'
-        dco = zlib.decompressobj()
-        assert dco.eof == False
-        dco.decompress(x[:-5])
-        assert dco.eof == False
-        dco.flush()
-        assert dco.eof == False
-
     def test_decompress_incomplete_stream(self):
         import zlib
         # This is 'foo', deflated
@@ -246,23 +222,23 @@ class AppTestZlib(object):
         It should show up in the unused_data attribute.
         """
         d = self.zlib.decompressobj()
-        s = d.decompress(self.compressed + b'extrastuff', 0)
+        s = d.decompress(self.compressed + 'extrastuff', 0)
         assert s == self.expanded
-        assert d.unused_data == b'extrastuff'
-        assert d.flush() == b''
-        assert d.unused_data == b'extrastuff'
+        assert d.unused_data == 'extrastuff'
+        assert d.flush() == ''
+        assert d.unused_data == 'extrastuff'
         # try again with several decompression steps
         d = self.zlib.decompressobj()
         s1 = d.decompress(self.compressed[:10])
-        assert d.unused_data == b''
+        assert d.unused_data == ''
         s2 = d.decompress(self.compressed[10:-3])
-        assert d.unused_data == b''
-        s3 = d.decompress(self.compressed[-3:] + b'spam' * 100)
-        assert d.unused_data == b'spam' * 100
+        assert d.unused_data == ''
+        s3 = d.decompress(self.compressed[-3:] + 'spam' * 100)
+        assert d.unused_data == 'spam' * 100
         assert s1 + s2 + s3 == self.expanded
-        s4 = d.decompress(b'egg' * 50)
-        assert d.unused_data == (b'spam' * 100) + (b'egg' * 50)
-        assert s4 == b''
+        s4 = d.decompress('egg' * 50)
+        assert d.unused_data == ('spam' * 100) + ('egg' * 50)
+        assert s4 == ''
 
     def test_max_length(self):
         """
@@ -279,6 +255,9 @@ class AppTestZlib(object):
 
     def test_max_length_large(self):
         import sys
+        if sys.version_info < (2, 7, 13):
+            skip("passing a potentially 64-bit int as max_length is not "
+                 "supported before 2.7.13")
         d = self.zlib.decompressobj()
         assert d.decompress(self.compressed, sys.maxsize) == self.expanded
 
@@ -286,23 +265,23 @@ class AppTestZlib(object):
         """
         We should be able to pass buffer objects instead of strings.
         """
-        assert self.zlib.crc32(memoryview(b'hello, world.')) == 3358036098
-        assert self.zlib.adler32(memoryview(b'hello, world.')) == 571147447
+        assert self.zlib.crc32(buffer('hello, world.')) == -936931198
+        assert self.zlib.adler32(buffer('hello, world.')) == 571147447
 
         compressor = self.zlib.compressobj()
-        bytes = compressor.compress(memoryview(self.expanded))
+        bytes = compressor.compress(buffer(self.expanded))
         bytes += compressor.flush()
         assert bytes == self.compressed
 
         decompressor = self.zlib.decompressobj()
-        bytes = decompressor.decompress(memoryview(self.compressed))
+        bytes = decompressor.decompress(buffer(self.compressed))
         bytes += decompressor.flush()
         assert bytes == self.expanded
 
-        bytes = self.zlib.compress(memoryview(self.expanded))
+        bytes = self.zlib.compress(buffer(self.expanded))
         assert bytes == self.compressed
 
-        bytes = self.zlib.decompress(memoryview(self.compressed))
+        bytes = self.zlib.decompress(buffer(self.compressed))
         assert bytes == self.expanded
 
     def test_flush_with_freed_input(self):
@@ -319,48 +298,6 @@ class AppTestZlib(object):
         assert dco.unused_data == b''
         assert dco.unconsumed_tail == b''
 
-    def test_dictionary(self):
-        l = self.LICENSE
-        # Build a simulated dictionary out of the words in LICENSE.
-        words = l.split()
-        zdict = b''.join(set(words))
-        # Use it to compress LICENSE.
-        co = self.zlib.compressobj(zdict=zdict)
-        cd = co.compress(l) + co.flush()
-        # Verify that it will decompress with the dictionary.
-        dco = self.zlib.decompressobj(zdict=zdict)
-        assert dco.decompress(cd) + dco.flush() == l
-        # Verify that it fails when not given the dictionary.
-        dco = self.zlib.decompressobj()
-        raises(self.zlib.error, dco.decompress, cd)
-
-    def test_dictionary_streaming(self):
-        # This simulates the reuse of a compressor object for compressing
-        # several separate data streams.
-        co = self.zlib.compressobj(zdict=self.LICENSE)
-        do = self.zlib.decompressobj(zdict=self.LICENSE)
-        piece = self.LICENSE[1000:1500]
-        d0 = co.compress(piece) + co.flush(self.zlib.Z_SYNC_FLUSH)
-        d1 = co.compress(piece[100:]) + co.flush(self.zlib.Z_SYNC_FLUSH)
-        d2 = co.compress(piece[:-100]) + co.flush(self.zlib.Z_SYNC_FLUSH)
-        assert do.decompress(d0) == piece
-        do.decompress(d1) == piece[100:]
-        do.decompress(d2) == piece[:-100]
-
-    def test_version(self):
-        zlib = self.zlib
-        assert zlib.ZLIB_VERSION[0] == zlib.ZLIB_RUNTIME_VERSION[0]
-
-    # CPython issue27164
-    def test_decompress_raw_with_dictionary(self):
-        zlib = self.zlib
-        zdict = b'abcdefghijklmnopqrstuvwxyz'
-        co = zlib.compressobj(wbits=-zlib.MAX_WBITS, zdict=zdict)
-        comp = co.compress(zdict) + co.flush()
-        dco = zlib.decompressobj(wbits=-zlib.MAX_WBITS, zdict=zdict)
-        uncomp = dco.decompress(comp) + dco.flush()
-        assert zdict == uncomp
-
     def test_decompress_copy(self):
         decompressor = self.zlib.decompressobj()
         d1 = decompressor.decompress(self.compressed[:10])
@@ -391,7 +328,7 @@ class AppTestZlib(object):
         assert decompressor.copy().unconsumed_tail == unconsumed_tail
 
         decompressor.decompress(decompressor.unconsumed_tail)
-        decompressor.decompress(b"xxx")
+        decompressor.decompress("xxx")
         unused_data = decompressor.unused_data
         assert unused_data
         assert decompressor.copy().unused_data == unused_data
@@ -408,7 +345,7 @@ class AppTestZlib(object):
 
         assert (d1 + from_copy) == (d1 + from_compressor)
 
-    @py.test.mark.skipif(rzlib.ZLIB_VERSION in ('1.2.7', '1.2.8', '1.2.3'), reason='does not error check')
+    @py.test.mark.skipif(rzlib.ZLIB_VERSION in ('1.2.8', '1.2.3'), reason='does not error check')
     def test_cannot_copy_compressor_with_stream_in_inconsistent_state(self):
         if self.runappdirect: skip("can't run with -A")
         compressor = self.zlib.compressobj()
