@@ -44,7 +44,7 @@ def LOOKUP_METHOD(f, nameindex, *ignored):
 
     w_type = space.type(w_obj)
     if w_type.has_object_getattribute():
-        name = space.str_w(w_name)
+        name = space.text_w(w_name)
         # bit of a mess to use these internal functions, but it allows the
         # mapdict caching below to work without an additional lookup
         version_tag = w_type.version_tag()
@@ -80,14 +80,14 @@ def LOOKUP_METHOD(f, nameindex, *ignored):
     if w_value is None:
         w_value = space.getattr(w_obj, w_name)
     f.pushvalue(w_value)
-    f.pushvalue(None)
+    f.pushvalue_none()
 
 @jit.unroll_safe
 def CALL_METHOD(f, oparg, *ignored):
     # opargs contains the arg, and kwarg count, excluding the implicit 'self'
     n_args = oparg & 0xff
     n_kwargs = (oparg >> 8) & 0xff
-    w_self = f.peekvalue(n_args + (2 * n_kwargs))
+    w_self = f.peekvalue_maybe_none(n_args + (2 * n_kwargs))
     n = n_args + (w_self is not None)
 
     if not n_kwargs:
@@ -106,7 +106,7 @@ def CALL_METHOD(f, oparg, *ignored):
                 break
             w_value = f.popvalue()
             w_key = f.popvalue()
-            key = f.space.str_w(w_key)
+            key = f.space.text_w(w_key)
             keywords[n_kwargs] = key
             keywords_w[n_kwargs] = w_value
 
@@ -115,7 +115,7 @@ def CALL_METHOD(f, oparg, *ignored):
                 arguments, keywords, keywords_w, None, None,
                 methodcall=w_self is not None)
         if w_self is None:
-            f.popvalue()    # removes w_self, which is None
+            f.popvalue_maybe_none()    # removes w_self, which is None
         w_callable = f.popvalue()
         if f.get_is_being_profiled() and function.is_builtin_code(w_callable):
             w_result = f.space.call_args_and_c_profile(f, w_callable, args)
@@ -138,6 +138,6 @@ def call_method_opt(space, w_obj, methname, *arg_w):
                 # fast method path: a function object in the class,
                 # nothing in the instance
                 return space.call_function(w_descr, w_obj, *arg_w)
-    w_name = space.wrap(methname)
+    w_name = space.newtext(methname)
     w_meth = space.getattr(w_obj, w_name)
     return space.call_function(w_meth, *arg_w)
