@@ -2,15 +2,16 @@
 import math
 import sys
 
-from rpython.rlib import jit
+from rpython.rlib import jit, rutf8
 from rpython.rlib.objectmodel import specialize
-from rpython.rlib.rarithmetic import INT_MAX
+from rpython.rlib.rarithmetic import INT_MAX, r_uint
 from rpython.rlib.rfloat import DTSF_ALT, formatd
-from rpython.rlib.rstring import StringBuilder, UnicodeBuilder
+from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.tool.sourcetools import func_with_new_name
 
 from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.unicodehelper import check_ascii_or_raise
 
 
 class BaseStringFormatter(object):
@@ -154,18 +155,15 @@ def make_formatter_subclass(do_unicode):
     # to build two subclasses of the BaseStringFormatter class,
     # each one getting its own subtle differences and RPython types.
 
-    if do_unicode:
-        const = unicode
-    else:
-        const = str
-
     class StringFormatter(BaseStringFormatter):
         def __init__(self, space, fmt, values_w, w_valuedict):
             BaseStringFormatter.__init__(self, space, values_w, w_valuedict)
-            self.fmt = fmt    # either a string or a unicode
+            self.fmt = fmt    # always a string, if unicode, utf8 encoded
 
         def peekchr(self):
-            # return the 'current' character
+            # Return the 'current' character. Note that this returns utf8
+            # encoded part, but this is ok since we only need one-character
+            # comparisons
             try:
                 return self.fmt[self.fmtpos]
             except IndexError:
@@ -202,7 +200,8 @@ def make_formatter_subclass(do_unicode):
             if self.w_valuedict is None:
                 raise oefmt(space.w_TypeError, "format requires a mapping")
             if do_unicode:
-                w_key = space.newunicode(key)
+                lgt = rutf8.check_utf8(key, True)
+                w_key = space.newutf8(key, lgt)
             else:
                 w_key = space.newbytes(key)
             return space.getitem(self.w_valuedict, w_key)
@@ -288,10 +287,7 @@ def make_formatter_subclass(do_unicode):
         @jit.look_inside_iff(lambda self: jit.isconstant(self.fmt))
         def format(self):
             lgt = len(self.fmt) + 4 * len(self.values_w) + 10
-            if do_unicode:
-                result = UnicodeBuilder(lgt)
-            else:
-                result = StringBuilder(lgt)
+            result = StringBuilder(lgt)
             self.result = result
             while True:
                 # fast path: consume as many characters as possible
@@ -312,8 +308,17 @@ def make_formatter_subclass(do_unicode):
                 c = self.peekchr()
                 self.forward()
                 if c == '%':
-                    self.std_wp(const('%'))
+                    self.result.append('%')
                     continue
+
+                # first check whether it's a invalid char, *then* call
+                # nextinputvalue, otherwise the error generated by
+                # nextinputvalue can cover that of unknown_fmtchar
+                for c1 in FORMATTER_CHARS:
+                    if c == c1:
+                        break
+                else:
+                    self.unknown_fmtchar()
                 if w_value is None:
                     w_value = self.nextinputvalue()
 
@@ -326,30 +331,35 @@ def make_formatter_subclass(do_unicode):
                         do_fmt = getattr(self, 'fmt_' + c1)
                         do_fmt(w_value)
                         break
-                else:
-                    self.unknown_fmtchar()
 
             self.checkconsumed()
             return result.build()
 
         def unknown_fmtchar(self):
             space = self.space
-            c = self.fmt[self.fmtpos - 1]
-            w_s = space.newunicode(c) if do_unicode else space.newbytes(c)
+            if do_unicode:
+                cp = rutf8.codepoint_at_pos(self.fmt, self.fmtpos - 1)
+                pos = rutf8.codepoints_in_utf8(self.fmt, 0, self.fmtpos - 1)
+                w_s = space.newutf8(rutf8.unichr_as_utf8(r_uint(cp),
+                                                  allow_surrogates=True), 1)
+            else:
+                cp = ord(self.fmt[self.fmtpos - 1])
+                pos = self.fmtpos - 1
+                w_s = space.newbytes(chr(cp))
             raise oefmt(space.w_ValueError,
                         "unsupported format character %R (%s) at index %d",
-                        w_s, hex(ord(c)), self.fmtpos - 1)
+                        w_s, hex(cp), pos)
 
-        @specialize.argtype(1)
-        def std_wp(self, r):
+        @specialize.arg(2)
+        def std_wp(self, r, is_string=False):
             length = len(r)
-            if do_unicode and isinstance(r, str):
+            if do_unicode and is_string:
                 # convert string to unicode using the default encoding
-                r = self.space.unicode_w(self.space.newbytes(r))
+                r = self.space.utf8_w(self.space.newbytes(r))
             prec = self.prec
             if prec == -1 and self.width == 0:
                 # fast path
-                self.result.append(const(r))
+                self.result.append(r)
                 return
             if prec >= 0 and prec < length:
                 length = prec   # ignore the end of the string if too long
@@ -359,12 +369,12 @@ def make_formatter_subclass(do_unicode):
                 padding = 0
             assert padding >= 0
             if not self.f_ljust and padding > 0:
-                result.append_multiple_char(const(' '), padding)
+                result.append_multiple_char(' ', padding)
                 # add any padding at the left of 'r'
                 padding = 0
             result.append_slice(r, 0, length)       # add 'r' itself
             if padding > 0:
-                result.append_multiple_char(const(' '), padding)
+                result.append_multiple_char(' ', padding)
             # add any remaining padding at the right
 
         def std_wp_number(self, r, prefix=''):
@@ -376,10 +386,10 @@ def make_formatter_subclass(do_unicode):
                 # result.append(), and no startswith() if not f_sign and
                 # not f_blank).
                 if self.f_sign and not r.startswith('-'):
-                    result.append(const('+'))
+                    result.append('+')
                 elif self.f_blank and not r.startswith('-'):
-                    result.append(const(' '))
-                result.append(const(r))
+                    result.append(' ')
+                result.append(r)
                 return
             # add a '+' or ' ' sign if necessary
             sign = r.startswith('-')
@@ -406,18 +416,18 @@ def make_formatter_subclass(do_unicode):
 
             assert padding >= 0
             if padnumber == '>':
-                result.append_multiple_char(const(' '), padding)
+                result.append_multiple_char(' ', padding)
                 # pad with spaces on the left
             if sign:
-                result.append(const(r[0]))        # the sign
-            result.append(const(prefix))               # the prefix
+                result.append(r[0])        # the sign
+            result.append(prefix)               # the prefix
             if padnumber == '0':
-                result.append_multiple_char(const('0'), padding)
+                result.append_multiple_char('0', padding)
                 # pad with zeroes
-            result.append_slice(const(r), int(sign), len(r))
+            result.append_slice(r, int(sign), len(r))
             # the rest of the number
             if padnumber == '<':           # spaces on the right
-                result.append_multiple_char(const(' '), padding)
+                result.append_multiple_char(' ', padding)
 
         def string_formatting(self, w_value):
             space = self.space
@@ -426,17 +436,17 @@ def make_formatter_subclass(do_unicode):
                 raise oefmt(space.w_TypeError,
                             "operand does not support unary str")
             w_result = space.get_and_call_function(w_impl, w_value)
-            if space.isinstance_w(w_result,
-                                              space.w_unicode):
+            if space.isinstance_w(w_result, space.w_unicode):
                 raise NeedUnicodeFormattingError
             return space.bytes_w(w_result)
 
         def fmt_s(self, w_value):
             space = self.space
-            got_unicode = space.isinstance_w(w_value,
-                                                         space.w_unicode)
+            got_unicode = space.isinstance_w(w_value, space.w_unicode)
             if not do_unicode:
                 if got_unicode:
+                    # Make sure the format string is ascii encodable
+                    check_ascii_or_raise(space, self.fmt)
                     raise NeedUnicodeFormattingError
                 s = self.string_formatting(w_value)
             else:
@@ -445,43 +455,46 @@ def make_formatter_subclass(do_unicode):
                 else:
                     from pypy.objspace.std.unicodeobject import unicode_from_object
                     w_value = unicode_from_object(space, w_value)
-                s = space.unicode_w(w_value)
-            self.std_wp(s)
+                s = space.utf8_w(w_value)
+            self.std_wp(s, False)
 
         def fmt_r(self, w_value):
-            self.std_wp(self.space.text_w(self.space.repr(w_value)))
+            self.std_wp(self.space.text_w(self.space.repr(w_value)), True)
 
         def fmt_c(self, w_value):
             self.prec = -1     # just because
             space = self.space
             if space.isinstance_w(w_value, space.w_bytes):
+                if do_unicode:
+                    w_value = w_value.descr_decode(space, space.newtext('ascii'))
                 s = space.bytes_w(w_value)
                 if len(s) != 1:
                     raise oefmt(space.w_TypeError, "%c requires int or char")
-                self.std_wp(s)
+                self.std_wp(s, True)
             elif space.isinstance_w(w_value, space.w_unicode):
                 if not do_unicode:
                     raise NeedUnicodeFormattingError
-                ustr = space.unicode_w(w_value)
-                if len(ustr) != 1:
+                ustr = space.utf8_w(w_value)
+                if space.len_w(w_value) != 1:
                     raise oefmt(space.w_TypeError, "%c requires int or unichar")
-                self.std_wp(ustr)
+                self.std_wp(ustr, False)
             else:
                 n = space.int_w(w_value)
                 if do_unicode:
                     try:
-                        c = unichr(n)
-                    except ValueError:
+                        c = rutf8.unichr_as_utf8(r_uint(n),
+                                                 allow_surrogates=True)
+                    except rutf8.OutOfRange:
                         raise oefmt(space.w_OverflowError,
                                     "unicode character code out of range")
-                    self.std_wp(c)
+                    self.std_wp(c, False)
                 else:
                     try:
                         s = chr(n)
                     except ValueError:
                         raise oefmt(space.w_OverflowError,
                                     "character code not in range(256)")
-                    self.std_wp(s)
+                    self.std_wp(s, True)
 
     return StringFormatter
 
@@ -512,10 +525,12 @@ def format(space, w_fmt, values_w, w_valuedict, do_unicode):
             pass
         else:
             return space.newbytes(result)
-    fmt = space.unicode_w(w_fmt)
+    fmt = space.utf8_w(w_fmt)
     formatter = UnicodeFormatter(space, fmt, values_w, w_valuedict)
     result = formatter.format()
-    return space.newunicode(result)
+    # this can force strings, not sure if it's a problem or not
+    lgt = rutf8.codepoints_in_utf8(result)
+    return space.newutf8(result, lgt)
 
 def mod_format(space, w_format, w_values, do_unicode=False):
     if space.isinstance_w(w_values, space.w_tuple):
