@@ -168,3 +168,24 @@ def test_hash():
     assert hash(myHashClass()) == -2
     assert hash(myHashClass2()) == -2
     assert hash(myHashClass3()) == hash(-10**100)
+
+def test_issue3255():
+    class MagicCaller(object):
+        def __get__(self, *args):
+            raise Exception("should not be called")
+        def __call__(self, *args):
+            return lambda attr: attr
+    class Descriptor(object):
+        __get__ = MagicCaller()
+    class X(object):
+        __getattribute__ = Descriptor()
+    assert X().foo == "foo"
+
+def test_descr_funny_new():
+    class C(object):
+        @classmethod
+        def __new__(*args):
+            return args
+
+    assert C.__new__(1,2) == (C, 1, 2)
+    assert C(1,2) == (C, C, 1, 2)
