@@ -1,4 +1,3 @@
-import pytest
 import sys
 import py
 import pypy
@@ -149,15 +148,6 @@ def test_apptest_spaceconfig(testdir):
     result = testdir.runpytest(p)
     assert result.ret == 0
 
-def test_rename_module():
-    from pypy.tool.pytest.apptest import _rename_module
-    assert _rename_module("sys") == "sys"
-    if sys.platform == "win32":
-        assert _rename_module("_winreg") == "winreg"
-    assert _rename_module("struct") == "_struct"
-    assert _rename_module("operator") == "_operator"
-    assert _rename_module("signal") == "_signal"
-
 def test_apptest_skipif(testdir):
     setpypyconftest(testdir)
     p = testdir.makepyfile(apptest_raise="""
