@@ -1,9 +1,13 @@
 import sys
-from rpython.jit.metainterp.typesystem import deref, fieldType, arrayItem
 from rpython.rtyper.rclass import OBJECT
 from rpython.rtyper.lltypesystem import lltype, llmemory
 from rpython.translator.backendopt.graphanalyze import BoolGraphAnalyzer
 from rpython.tool.algo import bitstring
+from rpython.jit.metainterp.support import int2adr
+
+
+class UnsupportedFieldExc(Exception):
+    pass
 
 
 class EffectInfo(object):
@@ -27,6 +31,7 @@ class EffectInfo(object):
     OS_DICT_LOOKUP              = 4    # ll_dict_lookup
     OS_THREADLOCALREF_GET       = 5    # llop.threadlocalref_get
     OS_NOT_IN_TRACE             = 8    # for calls not recorded in the jit trace
+    OS_ARRAYMOVE                = 9    # "list.ll_arraymove"
     #
     OS_INT_PY_DIV               = 12   # python signed division (neg. corrected)
     OS_INT_UDIV                 = 13   # regular unsigned division
@@ -117,8 +122,7 @@ class EffectInfo(object):
                 can_invalidate=False,
                 call_release_gil_target=_NO_CALL_RELEASE_GIL_TARGET,
                 extradescrs=None,
-                can_collect=True,
-                call_shortcut=None):
+                can_collect=True):
         readonly_descrs_fields = frozenset_or_none(readonly_descrs_fields)
         readonly_descrs_arrays = frozenset_or_none(readonly_descrs_arrays)
         readonly_descrs_interiorfields = frozenset_or_none(
@@ -136,8 +140,7 @@ class EffectInfo(object):
                extraeffect,
                oopspecindex,
                can_invalidate,
-               can_collect,
-               call_shortcut)
+               can_collect)
         tgt_func, tgt_saveerr = call_release_gil_target
         if tgt_func:
             key += (object(),)    # don't care about caching in this case
@@ -192,13 +195,12 @@ class EffectInfo(object):
         result.oopspecindex = oopspecindex
         result.extradescrs = extradescrs
         result.call_release_gil_target = call_release_gil_target
-        result.call_shortcut = call_shortcut
         if result.check_can_raise(ignore_memoryerror=True):
             assert oopspecindex in cls._OS_CANRAISE
 
         if (result._write_descrs_arrays is not None and
             len(result._write_descrs_arrays) == 1):
-            # this is used only for ARRAYCOPY operations
+            # this is used only for ARRAYCOPY/ARRAYMOVE operations
             [result.single_write_descr_array] = result._write_descrs_arrays
         else:
             result.single_write_descr_array = None
@@ -278,8 +280,7 @@ def effectinfo_from_writeanalyze(effects, cpu,
                                  call_release_gil_target=
                                      EffectInfo._NO_CALL_RELEASE_GIL_TARGET,
                                  extradescr=None,
-                                 can_collect=True,
-                                 call_shortcut=None):
+                                 can_collect=True):
     from rpython.translator.backendopt.writeanalyze import top_set
     if effects is top_set or extraeffect == EffectInfo.EF_RANDOM_EFFECTS:
         readonly_descrs_fields = None
@@ -298,39 +299,45 @@ def effectinfo_from_writeanalyze(effects, cpu,
         write_descrs_interiorfields = []
 
         def add_struct(descrs_fields, (_, T, fieldname)):
-            T = deref(T)
+            T = T.TO
             if consider_struct(T, fieldname):
                 descr = cpu.fielddescrof(T, fieldname)
                 descrs_fields.append(descr)
 
         def add_array(descrs_arrays, (_, T)):
-            ARRAY = deref(T)
+            ARRAY = T.TO
             if consider_array(ARRAY):
                 descr = cpu.arraydescrof(ARRAY)
                 descrs_arrays.append(descr)
 
         def add_interiorfield(descrs_interiorfields, (_, T, fieldname)):
-            T = deref(T)
+            T = T.TO
             if not isinstance(T, lltype.Array):
                 return # let's not consider structs for now
             if not consider_array(T):
                 return
             if getattr(T.OF, fieldname) is lltype.Void:
                 return
-            descr = cpu.interiorfielddescrof(T, fieldname)
+            try:
+                descr = cpu.interiorfielddescrof(T, fieldname)
+            except UnsupportedFieldExc:
+                return
             descrs_interiorfields.append(descr)
 
         # a read or a write to an interiorfield, inside an array of
         # structs, is additionally recorded as a read or write of
         # the array itself
-        extraef = set()
+        extraef = list()
         for tup in effects:
             if tup[0] == "interiorfield" or tup[0] == "readinteriorfield":
-                T = deref(tup[1])
+                T = tup[1].TO
                 if isinstance(T, lltype.Array) and consider_array(T):
-                    extraef.add((tup[0].replace("interiorfield", "array"),
-                                 tup[1]))
-        effects |= extraef
+                    val = (tup[0].replace("interiorfield", "array"),
+                                 tup[1])
+                    if val not in effects:
+                        extraef.append(val)
+        # preserve order in the added effects issue #2984
+        effects = tuple(effects) + tuple(extraef)
 
         for tup in effects:
             if tup[0] == "struct":
@@ -368,11 +375,10 @@ def effectinfo_from_writeanalyze(effects, cpu,
                       can_invalidate,
                       call_release_gil_target,
                       extradescr,
-                      can_collect,
-                      call_shortcut)
+                      can_collect)
 
 def consider_struct(TYPE, fieldname):
-    if fieldType(TYPE, fieldname) is lltype.Void:
+    if getattr(TYPE, fieldname) is lltype.Void:
         return False
     if not isinstance(TYPE, lltype.GcStruct): # can be a non-GC-struct
         return False
@@ -384,7 +390,7 @@ def consider_struct(TYPE, fieldname):
     return True
 
 def consider_array(ARRAY):
-    if arrayItem(ARRAY) is lltype.Void:
+    if ARRAY.OF is lltype.Void:
         return False
     if not isinstance(ARRAY, lltype.GcArray): # can be a non-GC-array
         return False
@@ -392,24 +398,6 @@ def consider_array(ARRAY):
 
 # ____________________________________________________________
 
-
-class CallShortcut(object):
-    def __init__(self, argnum, fielddescr):
-        self.argnum = argnum
-        self.fielddescr = fielddescr
-
-    def __eq__(self, other):
-        return (isinstance(other, CallShortcut) and
-                self.argnum == other.argnum and
-                self.fielddescr == other.fielddescr)
-    def __ne__(self, other):
-        return not (self == other)
-    def __hash__(self):
-        return hash((self.argnum, self.fielddescr))
-
-# ____________________________________________________________
-
-
 class VirtualizableAnalyzer(BoolGraphAnalyzer):
     def analyze_simple_operation(self, op, graphinfo):
         return op.opname in ('jit_force_virtualizable',
@@ -459,9 +447,8 @@ class CallInfoCollection(object):
             return (None, 0)
 
     def _funcptr_for_oopspec_memo(self, oopspecindex):
-        from rpython.jit.codewriter import heaptracker
         _, func_as_int = self.callinfo_for_oopspec(oopspecindex)
-        funcadr = heaptracker.int2adr(func_as_int)
+        funcadr = int2adr(func_as_int)
         return funcadr.ptr
     _funcptr_for_oopspec_memo._annspecialcase_ = 'specialize:memo'
 
