@@ -196,7 +196,7 @@ def elidable_promote(promote_args='all'):
         d = {"_orig_func_unlikely_name": func, "hint": hint}
         exec py.code.Source("\n".join(code)).compile() in d
         result = d["f"]
-        result.func_name = func.func_name + "_promote"
+        result.__name__ = func.__name__ + "_promote"
         return result
     return decorator
 
@@ -427,7 +427,7 @@ def jit_callback(name):
         jitdriver = JitDriver(get_printable_location=get_printable_location,
                               greens=[], reds='auto', name=name)
         #
-        args = ','.join(['a%d' % i for i in range(func.func_code.co_argcount)])
+        args = ','.join(['a%d' % i for i in range(func.__code__.co_argcount)])
         source = """def callback_with_jitdriver(%(args)s):
                         jitdriver.jit_merge_point()
                         return real_callback(%(args)s)""" % locals()
@@ -910,7 +910,7 @@ class ExtEnterLeaveMarker(ExtRegistryEntry):
             return
         bk = self.bookkeeper
         s_func = bk.immutablevalue(func)
-        uniquekey = 'jitdriver.%s' % func.func_name
+        uniquekey = 'jitdriver.%s' % func.__name__
         args_s = args_s[:]
         for name in variables:
             if '.' not in name:
