@@ -1,9 +1,9 @@
 from weakref import WeakValueDictionary
 
 from rpython.annotator import model as annmodel
-from rpython.rlib import jit, types
+from rpython.rlib import jit, types, objectmodel, rgc
 from rpython.rlib.objectmodel import (malloc_zero_filled, we_are_translated,
-    _hash_string, keepalive_until_here, specialize, enforceargs)
+    ll_hash_string, keepalive_until_here, specialize, enforceargs, dont_inline)
 from rpython.rlib.signature import signature
 from rpython.rlib.rarithmetic import ovfcheck
 from rpython.rtyper.error import TyperError
@@ -44,11 +44,13 @@ def new_malloc(TP, name):
 mallocstr = new_malloc(STR, 'mallocstr')
 mallocunicode = new_malloc(UNICODE, 'mallocunicode')
 
+@specialize.memo()
 def emptystrfun():
-    return emptystr
+    return string_repr.convert_const("")
 
+@specialize.memo()
 def emptyunicodefun():
-    return emptyunicode
+    return unicode_repr.convert_const(u'')
 
 def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     @specialize.arg(0)
@@ -88,6 +90,17 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
         ll_assert(srcstart + length <= len(src.chars), "copystrc: src ovf")
         ll_assert(dststart >= 0, "copystrc: negative dststart")
         ll_assert(dststart + length <= len(dst.chars), "copystrc: dst ovf")
+        #
+        # If the 'split_gc_address_space' option is set, we must copy
+        # manually, character-by-character
+        if rgc.must_split_gc_address_space():
+            i = 0
+            while i < length:
+                dst.chars[dststart + i] = src.chars[srcstart + i]
+                i += 1
+            return
+        #  
+        #
         # from here, no GC operations can happen
         asrc = _get_raw_buf(SRC_TP, src, srcstart)
         adst = _get_raw_buf(DST_TP, dst, dststart)
@@ -108,6 +121,16 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
         """
         # xxx Warning: same note as above apply: don't do this at home
         assert length >= 0
+        #
+        # If the 'split_gc_address_space' option is set, we must copy
+        # manually, character-by-character
+        if rgc.must_split_gc_address_space():
+            i = 0
+            while i < length:
+                ptrdst[i] = src.chars[srcstart + i]
+                i += 1
+            return
+        #
         # from here, no GC operations can happen
         asrc = _get_raw_buf(SRC_TP, src, srcstart)
         adst = llmemory.cast_ptr_to_adr(ptrdst)
@@ -124,6 +147,16 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     def copy_raw_to_string(ptrsrc, dst, dststart, length):
         # xxx Warning: same note as above apply: don't do this at home
         assert length >= 0
+        #
+        # If the 'split_gc_address_space' option is set, we must copy
+        # manually, character-by-character
+        if rgc.must_split_gc_address_space():
+            i = 0
+            while i < length:
+                dst.chars[dststart + i] = ptrsrc[i]
+                i += 1
+            return
+        #
         # from here, no GC operations can happen
         adst = _get_raw_buf(SRC_TP, dst, dststart)
         asrc = llmemory.cast_ptr_to_adr(ptrsrc)
@@ -136,15 +169,19 @@ def _new_copy_contents_fun(SRC_TP, DST_TP, CHAR_TP, name):
     copy_raw_to_string = func_with_new_name(copy_raw_to_string,
                                               'copy_raw_to_%s' % name)
 
-    return copy_string_to_raw, copy_raw_to_string, copy_string_contents
+    return (copy_string_to_raw, copy_raw_to_string, copy_string_contents,
+            _get_raw_buf)
 
 (copy_string_to_raw,
  copy_raw_to_string,
- copy_string_contents) = _new_copy_contents_fun(STR, STR, Char, 'string')
+ copy_string_contents,
+ _get_raw_buf_string) = _new_copy_contents_fun(STR, STR, Char, 'string')
 
 (copy_unicode_to_raw,
  copy_raw_to_unicode,
- copy_unicode_contents) = _new_copy_contents_fun(UNICODE, UNICODE, UniChar, 'unicode')
+ copy_unicode_contents,
+ _get_raw_buf_unicode) = _new_copy_contents_fun(UNICODE, UNICODE, UniChar,
+                                                'unicode')
 
 CONST_STR_CACHE = WeakValueDictionary()
 CONST_UNICODE_CACHE = WeakValueDictionary()
@@ -371,23 +408,23 @@ class LLHelpers(AbstractLLHelpers):
     @staticmethod
     def ll_strhash(s):
         if s:
-            return LLHelpers._ll_strhash(s)
+            return jit.conditional_call_elidable(s.hash,
+                                                 LLHelpers._ll_strhash, s)
         else:
             return 0
 
     @staticmethod
-    @jit.elidable
-    @jit.call_shortcut
+    @dont_inline
+    @jit.dont_look_inside
     def _ll_strhash(s):
         # unlike CPython, there is no reason to avoid to return -1
         # but our malloc initializes the memory to zero, so we use zero as the
-        # special non-computed-yet value.
-        x = s.hash
+        # special non-computed-yet value.  Also, jit.conditional_call_elidable
+        # always checks for zero, for now.
+        x = ll_hash_string(s)
         if x == 0:
-            x = _hash_string(s.chars)
-            if x == 0:
-                x = 29872897
-            s.hash = x
+            x = 29872897
+        s.hash = x
         return x
 
     @staticmethod
@@ -396,6 +433,7 @@ class LLHelpers(AbstractLLHelpers):
 
     @staticmethod
     def ll_strfasthash(s):
+        ll_assert(s.hash != 0, "ll_strfasthash: hash==0")
         return s.hash     # assumes that the hash is already computed
 
     @staticmethod
@@ -428,10 +466,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos < rpos and s.chars[lpos] == ch:
+            while lpos <= rpos and s.chars[lpos] == ch:
                 lpos += 1
         if right:
-            while lpos < rpos + 1 and s.chars[rpos] == ch:
+            while lpos <= rpos and s.chars[rpos] == ch:
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -449,10 +487,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos < rpos and s.chars[lpos].isspace():
+            while lpos <= rpos and s.chars[lpos].isspace():
                 lpos += 1
         if right:
-            while lpos < rpos + 1 and s.chars[rpos].isspace():
+            while lpos <= rpos and s.chars[rpos].isspace():
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -470,10 +508,10 @@ class LLHelpers(AbstractLLHelpers):
         lpos = 0
         rpos = s_len - 1
         if left:
-            while lpos < rpos and LLHelpers.ll_contains(s2, s.chars[lpos]):
+            while lpos <= rpos and LLHelpers.ll_contains(s2, s.chars[lpos]):
                 lpos += 1
         if right:
-            while lpos < rpos + 1 and LLHelpers.ll_contains(s2, s.chars[rpos]):
+            while lpos <= rpos and LLHelpers.ll_contains(s2, s.chars[rpos]):
                 rpos -= 1
         if rpos < lpos:
             return s.empty()
@@ -733,6 +771,11 @@ class LLHelpers(AbstractLLHelpers):
         count = 0
         n = end - start
         m = len(s2.chars)
+        tp = typeOf(s1)
+        if tp == string_repr.lowleveltype or tp == Char:
+            NUL = '\0'
+        else:
+            NUL = u'\0'
 
         if m == 0:
             if mode == FAST_COUNT:
@@ -777,7 +820,7 @@ class LLHelpers(AbstractLLHelpers):
                     if i + m < len(s1.chars):
                         c = s1.chars[i + m]
                     else:
-                        c = '\0'
+                        c = NUL
                     if not bloom(mask, c):
                         i += m
                     else:
@@ -786,7 +829,7 @@ class LLHelpers(AbstractLLHelpers):
                     if i + m < len(s1.chars):
                         c = s1.chars[i + m]
                     else:
-                        c = '\0'
+                        c = NUL
                     if not bloom(mask, c):
                         i += m
         else:
@@ -1228,6 +1271,16 @@ class LLHelpers(AbstractLLHelpers):
         SRC = typeOf(src).TO     # STR or UNICODE
         DST = typeOf(dst).TO     # GcArray
         assert DST.OF is SRC.chars.OF
+        #
+        # If the 'split_gc_address_space' option is set, we must copy
+        # manually, character-by-character
+        if rgc.must_split_gc_address_space():
+            i = 0
+            while i < length:
+                dst[i] = src.chars[i]
+                i += 1
+            return lst
+        #
         # from here, no GC operations can happen
         asrc = llmemory.cast_ptr_to_adr(src) + (
             llmemory.offsetof(SRC, 'chars') +
@@ -1254,7 +1307,8 @@ STR.become(GcStruct('rpy_string', ('hash',  Signed),
                               'gethash': LLHelpers.ll_strhash,
                               'length': LLHelpers.ll_length,
                               'find': LLHelpers.ll_find,
-                              'rfind': LLHelpers.ll_rfind}))
+                              'rfind': LLHelpers.ll_rfind},
+                    hints={'remove_hash': True}))
 UNICODE.become(GcStruct('rpy_unicode', ('hash', Signed),
                         ('chars', Array(UniChar, hints={'immutable': True})),
                         adtmeths={'malloc' : staticAdtMethod(mallocunicode),
@@ -1262,8 +1316,8 @@ UNICODE.become(GcStruct('rpy_unicode', ('hash', Signed),
                                   'copy_contents' : staticAdtMethod(copy_unicode_contents),
                                   'copy_contents_from_str' : staticAdtMethod(copy_unicode_contents),
                                   'gethash': LLHelpers.ll_strhash,
-                                  'length': LLHelpers.ll_length}
-                        ))
+                                  'length': LLHelpers.ll_length},
+                    hints={'remove_hash': True}))
 
 
 # TODO: make the public interface of the rstr module cleaner
@@ -1278,8 +1332,6 @@ unichar_repr = UniCharRepr()
 char_repr.ll = LLHelpers
 unichar_repr.ll = LLHelpers
 unicode_repr = UnicodeRepr()
-emptystr = string_repr.convert_const("")
-emptyunicode = unicode_repr.convert_const(u'')
 
 StringRepr.repr = string_repr
 UnicodeRepr.repr = unicode_repr
@@ -1338,14 +1390,6 @@ def ll_getnextindex(iter):
 string_repr.iterator_repr = StringIteratorRepr()
 unicode_repr.iterator_repr = UnicodeIteratorRepr()
 
-# these should be in rclass, but circular imports prevent (also it's
-# not that insane that a string constant is built in this file).
-
-instance_str_prefix = string_repr.convert_const("<")
-instance_str_infix  = string_repr.convert_const(" object at 0x")
-instance_str_suffix = string_repr.convert_const(">")
-
-null_str = string_repr.convert_const("NULL")
-
-unboxed_instance_str_prefix = string_repr.convert_const("<unboxed ")
-unboxed_instance_str_suffix = string_repr.convert_const(">")
+@specialize.memo()
+def conststr(s):
+    return string_repr.convert_const(s)
