@@ -17,12 +17,15 @@ from rpython.rlib.rarithmetic import r_uint, r_longlong, r_ulonglong, intmask
 #
 # Helpers for simple cases
 
-def eci_from_header(c_header_source, include_dirs=None):
+def eci_from_header(c_header_source, include_dirs=None, libraries=None):
     if include_dirs is None:
         include_dirs = []
+    if libraries is None:
+        libraries = []
     return ExternalCompilationInfo(
         post_include_bits=[c_header_source],
-        include_dirs=include_dirs
+        include_dirs=include_dirs,
+        libraries=libraries,
     )
 
 def getstruct(name, c_header_source, interesting_fields):
@@ -75,9 +78,10 @@ def getintegerfunctionresult(function, args=None, c_header_source='', includes=[
         CConfig._compilation_info_.includes = includes
     return configure(CConfig)['RESULT']
 
-def has(name, c_header_source, include_dirs=None):
+def has(name, c_header_source, include_dirs=None, libraries=None):
     class CConfig:
-        _compilation_info_ = eci_from_header(c_header_source, include_dirs)
+        _compilation_info_ = \
+            eci_from_header(c_header_source, include_dirs, libraries)
         HAS = Has(name)
     return configure(CConfig)['HAS']
 
@@ -88,11 +92,11 @@ def verify_eci(eci):
         _compilation_info_ = eci
         WORKS = Works()
     configure(CConfig)
-    
+
 def checkcompiles(expression, c_header_source, include_dirs=None):
     """Check if expression compiles. If not, returns False"""
     return has(expression, c_header_source, include_dirs)
-    
+
 def sizeof(name, eci, **kwds):
     class CConfig:
         _compilation_info_ = eci
@@ -107,15 +111,18 @@ def memory_alignment():
     fields is properly aligned."""
     global _memory_alignment
     if _memory_alignment is None:
-        S = getstruct('struct memory_alignment_test', """
-           struct memory_alignment_test {
-               double d;
-               void* p;
-           };
-        """, [])
-        result = S._hints['align']
-        assert result & (result-1) == 0, "not a power of two??"
-        _memory_alignment = result
+        if sys.platform == 'win32':
+            _memory_alignment = 4
+        else:
+            S = getstruct('struct memory_alignment_test', """
+               struct memory_alignment_test {
+                   double d;
+                   void* p;
+               };
+            """, [])
+            result = S._hints['align']
+            assert result & (result-1) == 0, "not a power of two??"
+            _memory_alignment = result
     return _memory_alignment
 _memory_alignment = None
 
@@ -124,38 +131,32 @@ _memory_alignment = None
 # General interface
 
 class ConfigResult:
-    def __init__(self, CConfig, info, entries):
-        self.CConfig = CConfig
-        self.result = {}
+    def __init__(self, eci, info):
+        self.eci = eci
         self.info = info
-        self.entries = entries
-        
+        self.result = {}
+
     def get_entry_result(self, entry):
         try:
             return self.result[entry]
         except KeyError:
             pass
-        name = self.entries[entry]
-        info = self.info[name]
+        info = self.info[entry]
         self.result[entry] = entry.build_result(info, self)
         return self.result[entry]
 
-    def get_result(self):
-        return dict([(name, self.result[entry])
-                     for entry, name in self.entries.iteritems()])
 
 class _CWriter(object):
     """ A simple class which aggregates config parts
     """
-    def __init__(self, CConfig):
+    def __init__(self, eci):
         self.path = uniquefilepath()
         self.f = self.path.open("w")
-        self.config = CConfig
+        self.eci = eci
 
     def write_header(self):
         f = self.f
-        CConfig = self.config
-        CConfig._compilation_info_.write_c_header(f)
+        self.eci.write_c_header(f)
         print >> f, C_HEADER
         print >> f
 
@@ -187,8 +188,7 @@ class _CWriter(object):
         self.start_main()
         self.f.write(question + "\n")
         self.close()
-        eci = self.config._compilation_info_
-        try_compile_cache([self.path], eci)
+        try_compile_cache([self.path], self.eci)
 
 def configure(CConfig, ignore_errors=False):
     """Examine the local system by running the C compiler.
@@ -198,52 +198,56 @@ def configure(CConfig, ignore_errors=False):
     """
     for attr in ['_includes_', '_libraries_', '_sources_', '_library_dirs_',
                  '_include_dirs_', '_header_']:
-        assert not hasattr(CConfig, attr), "Found legacy attribute %s on CConfig" % (attr,)
-    entries = []
+        assert not hasattr(CConfig, attr), \
+            "Found legacy attribute %s on CConfig" % attr
+
+    eci = CConfig._compilation_info_
+    entries = {}
     for key in dir(CConfig):
         value = getattr(CConfig, key)
         if isinstance(value, CConfigEntry):
-            entries.append((key, value))            
+            entries[key] = value
 
+    res = {}
     if entries:   # can be empty if there are only CConfigSingleEntries
-        writer = _CWriter(CConfig)
-        writer.write_header()
-        for key, entry in entries:
-            writer.write_entry(key, entry)
-
-        f = writer.f
-        writer.start_main()
-        for key, entry in entries:
-            writer.write_entry_main(key)
-        writer.close()
-
-        eci = CConfig._compilation_info_
-        infolist = list(run_example_code(writer.path, eci,
-                                         ignore_errors=ignore_errors))
-        assert len(infolist) == len(entries)
-
-        resultinfo = {}
-        resultentries = {}
-        for info, (key, entry) in zip(infolist, entries):
-            resultinfo[key] = info
-            resultentries[entry] = key
-
-        result = ConfigResult(CConfig, resultinfo, resultentries)
-        for name, entry in entries:
-            result.get_entry_result(entry)
-        res = result.get_result()
-    else:
-        res = {}
+        results = configure_entries(
+            entries.values(), eci, ignore_errors=ignore_errors)
+        for name, result in zip(entries, results):
+            res[name] = result
 
     for key in dir(CConfig):
         value = getattr(CConfig, key)
         if isinstance(value, CConfigSingleEntry):
-            writer = _CWriter(CConfig)
+            writer = _CWriter(eci)
             writer.write_header()
             res[key] = value.question(writer.ask_gcc)
 
     return res
 
+
+def configure_entries(entries, eci, ignore_errors=False):
+    writer = _CWriter(eci)
+    writer.write_header()
+    for i, entry in enumerate(entries):
+        writer.write_entry(str(i), entry)
+
+    writer.start_main()
+    for i, entry in enumerate(entries):
+        writer.write_entry_main(str(i))
+    writer.close()
+
+    infolist = list(run_example_code(
+        writer.path, eci, ignore_errors=ignore_errors))
+    assert len(infolist) == len(entries)
+
+    resultinfo = {}
+    for info, entry in zip(infolist, entries):
+        resultinfo[entry] = info
+
+    result = ConfigResult(eci, resultinfo)
+    for entry in entries:
+        yield result.get_entry_result(entry)
+
 # ____________________________________________________________
 
 
@@ -255,10 +259,11 @@ class Struct(CConfigEntry):
     """An entry in a CConfig class that stands for an externally
     defined structure.
     """
-    def __init__(self, name, interesting_fields, ifdef=None):
+    def __init__(self, name, interesting_fields, ifdef=None, adtmeths={}):
         self.name = name
         self.interesting_fields = interesting_fields
         self.ifdef = ifdef
+        self.adtmeths = adtmeths
 
     def prepare_code(self):
         if self.ifdef is not None:
@@ -305,7 +310,9 @@ class Struct(CConfigEntry):
                 offset = info['fldofs '  + fieldname]
                 size   = info['fldsize ' + fieldname]
                 sign   = info.get('fldunsigned ' + fieldname, False)
-                if (size, sign) != rffi.size_and_sign(fieldtype):
+                if is_array_nolength(fieldtype):
+                    pass       # ignore size and sign
+                elif (size, sign) != rffi.size_and_sign(fieldtype):
                     fieldtype = fixup_ctype(fieldtype, fieldname, (size, sign))
                 layout_addfield(layout, offset, fieldtype, fieldname)
 
@@ -333,7 +340,7 @@ class Struct(CConfigEntry):
         allfields = tuple(['c_' + name for name, _ in fields])
         padfields = tuple(padfields)
         name = self.name
-        eci = config_result.CConfig._compilation_info_
+        eci = config_result.eci
         padding_drop = PaddingDrop(name, allfields, padfields, eci)
         hints = {'align': info['align'],
                  'size': info['size'],
@@ -345,7 +352,7 @@ class Struct(CConfigEntry):
             name = name[7:]
         else:
             hints['typedef'] = True
-        kwds = {'hints': hints}
+        kwds = {'hints': hints, 'adtmeths': self.adtmeths}
         return rffi.CStruct(name, *fields, **kwds)
 
 class SimpleType(CConfigEntry):
@@ -356,7 +363,7 @@ class SimpleType(CConfigEntry):
         self.name = name
         self.ctype_hint = ctype_hint
         self.ifdef = ifdef
-        
+
     def prepare_code(self):
         if self.ifdef is not None:
             yield '#ifdef %s' % (self.ifdef,)
@@ -489,7 +496,7 @@ class DefinedConstantString(CConfigEntry):
     def prepare_code(self):
         yield '#ifdef %s' % self.macro
         yield 'int i;'
-        yield 'char *p = %s;' % self.name
+        yield 'const char *p = %s;' % self.name
         yield 'dump("defined", 1);'
         yield 'for (i = 0; p[i] != 0; i++ ) {'
         yield '  printf("value_%d: %d\\n", i, (int)(unsigned char)p[i]);'
@@ -535,10 +542,10 @@ class CConfigSingleEntry(object):
 class Has(CConfigSingleEntry):
     def __init__(self, name):
         self.name = name
-    
+
     def question(self, ask_gcc):
         try:
-            ask_gcc(self.name + ';')
+            ask_gcc('(void)' + self.name + ';')
             return True
         except CompilationError:
             return False
@@ -674,8 +681,14 @@ class Field(object):
     def __repr__(self):
         return '<field %s: %s>' % (self.name, self.ctype)
 
+def is_array_nolength(TYPE):
+    return isinstance(TYPE, lltype.Array) and TYPE._hints.get('nolength', False)
+
 def layout_addfield(layout, offset, ctype, prefix):
-    size = _sizeof(ctype)
+    if is_array_nolength(ctype):
+        size = len(layout) - offset    # all the rest of the struct
+    else:
+        size = _sizeof(ctype)
     name = prefix
     i = 0
     while name in layout:
@@ -697,7 +710,8 @@ def fixup_ctype(fieldtype, fieldname, expected_size_and_sign):
         size, _ = expected_size_and_sign
         return lltype.FixedSizeArray(fieldtype.OF, size/_sizeof(fieldtype.OF))
     raise TypeError("conflict between translating python and compiler field"
-                    " type %r for %r" % (fieldtype, fieldname))
+                    " type %r for symbol %r, expected size+sign %r" % (
+                        fieldtype, fieldname, expected_size_and_sign))
 
 def expose_value_as_rpython(value):
     if intmask(value) == value:
@@ -767,7 +781,7 @@ def configure_external_library(name, eci, configurations,
     On Windows, various configurations may be tried to compile the
     given eci object.  These configurations are a list of dicts,
     containing:
-    
+
     - prefix: if an absolute path, will prefix each include and
               library directories.  If a relative path, the external
               directory is searched for directories which names start
@@ -775,13 +789,13 @@ def configure_external_library(name, eci, configurations,
               chosen, and becomes the prefix.
 
     - include_dir: prefix + include_dir is added to the include directories
-    
+
     - library_dir: prefix + library_dir is added to the library directories
     """
 
     if sys.platform != 'win32':
         configurations = []
-    
+
     key = (name, eci)
     try:
         return _cache[key]
@@ -819,7 +833,7 @@ def configure_external_library(name, eci, configurations,
             # verify that this eci can be compiled
             try:
                 verify_eci(eci_lib)
-            except CompilationError, e:
+            except CompilationError as e:
                 last_error = e
             else:
                 _cache[key] = eci_lib
@@ -847,15 +861,20 @@ def configure_boehm(platform=None):
         # since config_external_library does not use a platform kwarg,
         # somehow using a platform kw arg make the merge fail in
         # config_external_library
-        platform = None    
+        platform = None
     else:
         library_dir = ''
-        libraries = ['gc', 'dl']
+        libraries = ['gc']
         includes=['gc/gc.h']
-    eci = ExternalCompilationInfo(
-        platform=platform,
-        includes=includes,
-        libraries=libraries,
+    try:
+        eci = ExternalCompilationInfo.from_pkg_config('bdw-gc')
+        eci.includes += tuple(includes)
+        return eci
+    except ImportError:
+        eci = ExternalCompilationInfo(
+            platform=platform,
+            includes=includes,
+            libraries=libraries,
         )
     return configure_external_library(
         'gc', eci,
@@ -864,7 +883,7 @@ def configure_boehm(platform=None):
 
 if __name__ == '__main__':
     doc = """Example:
-    
+
        rffi_platform.py  -h sys/types.h  -h netinet/in.h
                            'struct sockaddr_in'
                            sin_port  INT
