@@ -13,9 +13,9 @@ typedef struct RPyOpaque_ThreadLock {
 
 /* prototypes */
 RPY_EXTERN
-Signed RPyThreadStart(void (*func)(void));
+long RPyThreadStart(void (*func)(void));
 RPY_EXTERN
-Signed RPyThreadStartEx(void (*func)(void *), void *arg);
+long RPyThreadStartEx(void (*func)(void *), void *arg);
 RPY_EXTERN
 int RPyThreadLockInit(struct RPyOpaque_ThreadLock *lock);
 RPY_EXTERN
@@ -26,38 +26,21 @@ RPY_EXTERN
 RPyLockStatus RPyThreadAcquireLockTimed(struct RPyOpaque_ThreadLock *lock,
 					RPY_TIMEOUT_T timeout, int intr_flag);
 RPY_EXTERN
-Signed RPyThreadReleaseLock(struct RPyOpaque_ThreadLock *lock);
+long RPyThreadReleaseLock(struct RPyOpaque_ThreadLock *lock);
 RPY_EXTERN
-Signed RPyThreadGetStackSize(void);
+long RPyThreadGetStackSize(void);
 RPY_EXTERN
-Signed RPyThreadSetStackSize(Signed);
+long RPyThreadSetStackSize(long);
 #endif
 
 
 #ifdef _M_IA64
 /* On Itanium, use 'acquire' memory ordering semantics */
-
-#define pypy_lock_test_and_set(ptr, value) InterlockedExchangeAcquire64(ptr,value)
-#define pypy_compare_and_swap(ptr, oldval, newval)  \
-    (InterlockedCompareExchangeAcquire64(ptr, newval, oldval) == oldval)
-#define pypy_lock_release(ptr)             (*((volatile __int64 *)ptr) = 0)
-
+#define pypy_lock_test_and_set(ptr, value) InterlockedExchangeAcquire(ptr,value)
 #else
-#ifdef _WIN64
-/* 64-bit, not Itanium */
-
-#define pypy_lock_test_and_set(ptr, value) InterlockedExchange64(ptr, value)
-#define pypy_compare_and_swap(ptr, oldval, newval)  \
-    (InterlockedCompareExchange64(ptr, newval, oldval) == oldval)
-#define pypy_lock_release(ptr)             (*((volatile __int64 *)ptr) = 0)
-
-#else
-/* 32-bit */
-
 #define pypy_lock_test_and_set(ptr, value) InterlockedExchange(ptr, value)
-#define pypy_compare_and_swap(ptr, oldval, newval)  \
-    (InterlockedCompareExchange(ptr, newval, oldval) == oldval)
+#endif
 #define pypy_lock_release(ptr)             (*((volatile long *)ptr) = 0)
 
-#endif
-#endif
+#define pypy_compare_and_swap(ptr, oldval, newval)  \
+    (InterlockedCompareExchange(ptr, newval, oldval) == oldval)
