@@ -18,7 +18,9 @@ class Darwin(posix.BasePosix):
     shared_only = ()
 
     link_flags = (DARWIN_VERSION_MIN,)
-    cflags = ('-O3', '-fomit-frame-pointer', DARWIN_VERSION_MIN)
+    cflags = ('-O3',
+              '-fomit-frame-pointer',
+              DARWIN_VERSION_MIN,)
 
     so_ext = 'dylib'
     DEFAULT_CC = 'clang'
@@ -39,7 +41,9 @@ class Darwin(posix.BasePosix):
         # we get the basename of the executable we're trying to build.
         return (list(self.shared_only)
                 + ['-dynamiclib', '-install_name', '@rpath/' + target_basename,
-                   '-undefined', 'dynamic_lookup', '-flat_namespace']
+                   '-undefined', 'dynamic_lookup', '-flat_namespace',
+                   '-headerpad_max_install_names',
+                  ]
                 + args)
 
     def _include_dirs_for_libffi(self):
@@ -113,11 +117,15 @@ class Darwin(posix.BasePosix):
 
 class Darwin_PowerPC(Darwin):#xxx fixme, mwp
     name = "darwin_powerpc"
+    link_flags = Darwin.link_flags + ('-arch', 'ppc')
+    cflags = Darwin.cflags + ('-arch', 'ppc')
 
 class Darwin_i386(Darwin):
     name = "darwin_i386"
-    DEFAULT_CC = 'clang -arch i386'
+    link_flags = Darwin.link_flags + ('-arch', 'i386')
+    cflags = Darwin.cflags + ('-arch', 'i386')
 
 class Darwin_x86_64(Darwin):
     name = "darwin_x86_64"
-    DEFAULT_CC = 'clang -arch x86_64'
+    link_flags = Darwin.link_flags + ('-arch', 'x86_64')
+    cflags = Darwin.cflags + ('-arch', 'x86_64')
